@@ -30,8 +30,8 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-#ifndef YY_MAIN_CFG_GRAMMAR_H_INCLUDED
-# define YY_MAIN_CFG_GRAMMAR_H_INCLUDED
+#ifndef YY_MAIN_LIB_CFG_GRAMMAR_H_INCLUDED
+# define YY_MAIN_LIB_CFG_GRAMMAR_H_INCLUDED
 /* Debug traces.  */
 #ifndef YYDEBUG
 # define YYDEBUG 0
@@ -40,7 +40,7 @@
 extern int main_debug;
 #endif
 /* "%code requires" blocks.  */
-#line 25 "/home/algernon/zwa/work/syslog-ng-ose-3.4/syslog-ng-ose/lib/cfg-grammar.y" /* yacc.c:1909  */
+#line 25 "lib/cfg-grammar.y" /* yacc.c:1909  */
 
 
 /* this block is inserted into cfg-grammar.h, so it is included
@@ -51,9 +51,13 @@ extern int main_debug;
 /* YYSTYPE and YYLTYPE is defined by the lexer */
 #include "cfg-lexer.h"
 #include "afinter.h"
-#include "filter-expr-parser.h"
-#include "parser-expr-parser.h"
-#include "rewrite-expr-parser.h"
+#include "type-hinting.h"
+#include "filter/filter-expr-parser.h"
+#include "filter/filter-pipe.h"
+#include "parser/parser-expr-parser.h"
+#include "rewrite/rewrite-expr-parser.h"
+#include "logmatcher.h"
+#include "logthrdestdrv.h"
 
 /* uses struct declarations instead of the typedefs to avoid having to
  * include logreader/logwriter/driver.h, which defines the typedefs.  This
@@ -68,9 +72,16 @@ extern struct _FilePermOptions *last_file_perm_options;
 extern struct _MsgFormatOptions *last_msg_format_options;
 extern struct _LogDriver *last_driver;
 extern struct _LogParser *last_parser;
+extern struct _LogTemplateOptions *last_template_options;
+extern struct _LogTemplate *last_template;
+extern struct _ValuePairs *last_value_pairs;
+extern struct _ValuePairsTransformSet *last_vp_transset;
+extern struct _LogMatcherOptions *last_matcher_options;
+extern struct _HostResolveOptions *last_host_resolve_options;
+extern struct _StatsOptions *last_stats_options;
 
 
-#line 74 "cfg-grammar.h" /* yacc.c:1909  */
+#line 85 "lib/cfg-grammar.h" /* yacc.c:1909  */
 
 /* Token type.  */
 #ifndef YYTOKENTYPE
@@ -126,13 +137,14 @@ extern struct _LogParser *last_parser;
     KW_MARK_FREQ = 10071,
     KW_STATS_FREQ = 10072,
     KW_STATS_LEVEL = 10073,
-    KW_FLUSH_LINES = 10074,
-    KW_SUPPRESS = 10075,
-    KW_FLUSH_TIMEOUT = 10076,
-    KW_LOG_MSG_SIZE = 10077,
-    KW_FILE_TEMPLATE = 10078,
-    KW_PROTO_TEMPLATE = 10079,
-    KW_MARK_MODE = 10080,
+    KW_STATS_LIFETIME = 10074,
+    KW_FLUSH_LINES = 10075,
+    KW_SUPPRESS = 10076,
+    KW_FLUSH_TIMEOUT = 10077,
+    KW_LOG_MSG_SIZE = 10078,
+    KW_FILE_TEMPLATE = 10079,
+    KW_PROTO_TEMPLATE = 10080,
+    KW_MARK_MODE = 10081,
     KW_CHAIN_HOSTNAMES = 10090,
     KW_NORMALIZE_HOSTNAMES = 10091,
     KW_KEEP_HOSTNAME = 10092,
@@ -141,12 +153,14 @@ extern struct _LogParser *last_parser;
     KW_KEEP_TIMESTAMP = 10100,
     KW_USE_DNS = 10110,
     KW_USE_FQDN = 10111,
+    KW_CUSTOM_DOMAIN = 10112,
     KW_DNS_CACHE = 10120,
     KW_DNS_CACHE_SIZE = 10121,
     KW_DNS_CACHE_EXPIRE = 10130,
     KW_DNS_CACHE_EXPIRE_FAILED = 10131,
     KW_DNS_CACHE_HOSTS = 10132,
     KW_PERSIST_ONLY = 10140,
+    KW_USE_RCPTID = 10141,
     KW_TZ_CONVERT = 10150,
     KW_TS_FORMAT = 10151,
     KW_FRAC_DIGITS = 10152,
@@ -217,7 +231,9 @@ extern struct _LogParser *last_parser;
     KW_SHIFT = 10506,
     KW_REKEY = 10507,
     KW_ADD_PREFIX = 10508,
-    KW_REPLACE = 10509
+    KW_REPLACE_PREFIX = 10509,
+    KW_ON_ERROR = 10510,
+    KW_RETRIES = 10511
   };
 #endif
 /* Tokens.  */
@@ -270,13 +286,14 @@ extern struct _LogParser *last_parser;
 #define KW_MARK_FREQ 10071
 #define KW_STATS_FREQ 10072
 #define KW_STATS_LEVEL 10073
-#define KW_FLUSH_LINES 10074
-#define KW_SUPPRESS 10075
-#define KW_FLUSH_TIMEOUT 10076
-#define KW_LOG_MSG_SIZE 10077
-#define KW_FILE_TEMPLATE 10078
-#define KW_PROTO_TEMPLATE 10079
-#define KW_MARK_MODE 10080
+#define KW_STATS_LIFETIME 10074
+#define KW_FLUSH_LINES 10075
+#define KW_SUPPRESS 10076
+#define KW_FLUSH_TIMEOUT 10077
+#define KW_LOG_MSG_SIZE 10078
+#define KW_FILE_TEMPLATE 10079
+#define KW_PROTO_TEMPLATE 10080
+#define KW_MARK_MODE 10081
 #define KW_CHAIN_HOSTNAMES 10090
 #define KW_NORMALIZE_HOSTNAMES 10091
 #define KW_KEEP_HOSTNAME 10092
@@ -285,12 +302,14 @@ extern struct _LogParser *last_parser;
 #define KW_KEEP_TIMESTAMP 10100
 #define KW_USE_DNS 10110
 #define KW_USE_FQDN 10111
+#define KW_CUSTOM_DOMAIN 10112
 #define KW_DNS_CACHE 10120
 #define KW_DNS_CACHE_SIZE 10121
 #define KW_DNS_CACHE_EXPIRE 10130
 #define KW_DNS_CACHE_EXPIRE_FAILED 10131
 #define KW_DNS_CACHE_HOSTS 10132
 #define KW_PERSIST_ONLY 10140
+#define KW_USE_RCPTID 10141
 #define KW_TZ_CONVERT 10150
 #define KW_TS_FORMAT 10151
 #define KW_FRAC_DIGITS 10152
@@ -361,7 +380,9 @@ extern struct _LogParser *last_parser;
 #define KW_SHIFT 10506
 #define KW_REKEY 10507
 #define KW_ADD_PREFIX 10508
-#define KW_REPLACE 10509
+#define KW_REPLACE_PREFIX 10509
+#define KW_ON_ERROR 10510
+#define KW_RETRIES 10511
 
 /* Value type.  */
 
@@ -383,4 +404,4 @@ struct YYLTYPE
 
 int main_parse (CfgLexer *lexer, gpointer *dummy, gpointer arg);
 
-#endif /* !YY_MAIN_CFG_GRAMMAR_H_INCLUDED  */
+#endif /* !YY_MAIN_LIB_CFG_GRAMMAR_H_INCLUDED  */
