@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002-2012 BalaBit IT Ltd, Budapest, Hungary
+ * Copyright (c) 2002-2013 BalaBit IT Ltd, Budapest, Hungary
  * Copyright (c) 1998-2012 Balázs Scheidler
  *
  * This library is free software; you can redistribute it and/or
@@ -65,7 +65,8 @@ static CfgLexerKeyword main_keywords[] = {
   { "rekey",              KW_REKEY, 0x0303 },
   { "shift",              KW_SHIFT, 0x0303 },
   { "add_prefix",         KW_ADD_PREFIX, 0x0303 },
-  { "replace",            KW_REPLACE, 0x0303 },
+  { "replace",            KW_REPLACE_PREFIX, 0x0303, KWS_OBSOLETE, "replace_prefix" },
+  { "replace_prefix",     KW_REPLACE_PREFIX, 0x0304 },
 
   /* option items */
   { "flags",              KW_FLAGS },
@@ -74,6 +75,7 @@ static CfgLexerKeyword main_keywords[] = {
   { "mark",               KW_MARK_FREQ, 0, KWS_OBSOLETE, "mark_freq" },
   { "mark_mode",          KW_MARK_MODE, 0x0304 },
   { "stats_freq",         KW_STATS_FREQ },
+  { "stats_lifetime",     KW_STATS_LIFETIME, 0x0306 },
   { "stats_level",        KW_STATS_LEVEL },
   { "stats",              KW_STATS_FREQ, 0, KWS_OBSOLETE, "stats_freq" },
   { "flush_lines",        KW_FLUSH_LINES },
@@ -87,6 +89,7 @@ static CfgLexerKeyword main_keywords[] = {
   { "keep_hostname",      KW_KEEP_HOSTNAME },
   { "check_hostname",     KW_CHECK_HOSTNAME },
   { "bad_hostname",       KW_BAD_HOSTNAME },
+  { "custom_domain",      KW_CUSTOM_DOMAIN, 0x0306 },
   { "keep_timestamp",     KW_KEEP_TIMESTAMP },
   { "encoding",           KW_ENCODING, 0x0300 },
   { "ts_format",          KW_TS_FORMAT },
@@ -108,8 +111,10 @@ static CfgLexerKeyword main_keywords[] = {
   { "default_priority",   KW_DEFAULT_LEVEL, 0x0300 },
   { "default_facility",   KW_DEFAULT_FACILITY, 0x0300 },
   { "threaded",           KW_THREADED, 0x0303 },
+  { "use_rcptid",         KW_USE_RCPTID, 0x0306 },
 
   { "value",              KW_VALUE, 0x0300 },
+  { "values",             KW_VALUES },
 
   { "log_fifo_size",      KW_LOG_FIFO_SIZE },
   { "log_fetch_limit",    KW_LOG_FETCH_LIMIT },
@@ -131,6 +136,7 @@ static CfgLexerKeyword main_keywords[] = {
   { "dir_perm",           KW_DIR_PERM },
   { "template",           KW_TEMPLATE },
   { "template_escape",    KW_TEMPLATE_ESCAPE },
+  { "on_error",           KW_ON_ERROR },
   { "persist_only",       KW_PERSIST_ONLY },
   { "dns_cache_hosts",    KW_DNS_CACHE_HOSTS },
   { "dns_cache",          KW_DNS_CACHE },
@@ -138,6 +144,8 @@ static CfgLexerKeyword main_keywords[] = {
   { "dns_cache_expire",   KW_DNS_CACHE_EXPIRE },
   { "dns_cache_expire_failed", KW_DNS_CACHE_EXPIRE_FAILED },
 
+  { "retries",            KW_RETRIES, 0x0303 },
+
   /* filter items */
   { "type",               KW_TYPE, 0x0300 },
   { "tags",               KW_TAGS, 0x0300 },
@@ -232,7 +240,7 @@ report_syntax_error(CfgLexer *lexer, YYLTYPE *yylloc, const char *what, const ch
         {
           fprintf(stderr, "%c", buf[i] == '\t' ? '\t' : ' ');
         }
-      for (i = yylloc->first_column; i < yylloc->last_column; i++)
+      for (i = yylloc->first_column; (i == yylloc->first_column) || (i < yylloc->last_column); i++)
         fprintf(stderr, "^");
       fprintf(stderr, "\n");
     }
