@@ -23,10 +23,8 @@
  */
 
 #include "cfg-tree.h"
-#include "filter.h"
 #include "messages.h"
 #include "afinter.h"
-#include "logparser.h"
 #include "logmpx.h"
 
 #include <string.h>
@@ -156,6 +154,14 @@ log_expr_node_format_location(LogExprNode *self, gchar *buf, gsize buf_len)
   return buf;
 }
 
+EVTTAG *
+log_expr_node_location_tag(LogExprNode *self)
+{
+  gchar buf[128];
+
+  return evt_tag_str("location", log_expr_node_format_location(self, buf, sizeof(buf)));
+}
+
 /*
  * Set the list of children of a LogExprNode. It automatically updates
  * the children's "parent" pointers so that the tree can be traversed
@@ -447,10 +453,8 @@ cfg_tree_compile_single(CfgTree *self, LogExprNode *node,
       pipe = log_pipe_clone(pipe);
       if (!pipe)
         {
-          gchar buf[128];
-
           msg_error("Error cloning pipe into its reference point, probably the element in question is not meant to be used in this situation",
-                    evt_tag_str("location", log_expr_node_format_location(node, buf, sizeof(buf))),
+                    log_expr_node_location_tag(node),
                     NULL);
           goto error;
         }
@@ -486,12 +490,10 @@ cfg_tree_compile_reference(CfgTree *self, LogExprNode *node,
 
   if (!referenced_node)
     {
-      gchar buf[128];
-
       msg_error("Error resolving reference",
                 evt_tag_str("content", log_expr_node_get_content_name(node->content)),
                 evt_tag_str("name", node->name),
-                evt_tag_str("location", log_expr_node_format_location(node, buf, sizeof(buf))),
+                log_expr_node_location_tag(node),
                 NULL);
       goto error;
     }
@@ -515,7 +517,7 @@ cfg_tree_compile_reference(CfgTree *self, LogExprNode *node,
             sub_pipe_tail = referenced_node->aux;
           }
 
-        attach_pipe = log_pipe_new();
+        attach_pipe = log_pipe_new(self->cfg);
         g_ptr_array_add(self->initialized_pipes, attach_pipe);
 
         if (sub_pipe_tail)
@@ -526,7 +528,7 @@ cfg_tree_compile_reference(CfgTree *self, LogExprNode *node,
 
             if (!sub_pipe_tail->pipe_next)
               {
-                mpx = log_multiplexer_new(0);
+                mpx = log_multiplexer_new(0, self->cfg);
                 g_ptr_array_add(self->initialized_pipes, &mpx->super);
                 log_pipe_append(sub_pipe_tail, &mpx->super);
               }
@@ -567,7 +569,7 @@ cfg_tree_compile_reference(CfgTree *self, LogExprNode *node,
            our next chain
         */
 
-        mpx = log_multiplexer_new(0);
+        mpx = log_multiplexer_new(0, self->cfg);
         g_ptr_array_add(self->initialized_pipes, &mpx->super);
 
         if (sub_pipe_head)
@@ -723,17 +725,15 @@ cfg_tree_compile_sequence(CfgTree *self, LogExprNode *node,
 
           if (first_pipe)
             {
-              gchar buf[128];
-
               msg_error("Error compiling sequence, source-pipe follows a non-source one, please list source references/definitions first",
-                        evt_tag_str("location", log_expr_node_format_location(ep, buf, sizeof(buf))),
+                        log_expr_node_location_tag(ep),
                         NULL);
               goto error;
             }
 
           if (!source_join_pipe)
             {
-              source_join_pipe = last_pipe = log_pipe_new();
+              source_join_pipe = last_pipe = log_pipe_new(self->cfg);
               g_ptr_array_add(self->initialized_pipes, source_join_pipe);
             }
           log_pipe_append(sub_pipe_tail, source_join_pipe);
@@ -754,7 +754,7 @@ cfg_tree_compile_sequence(CfgTree *self, LogExprNode *node,
   if (!first_pipe && !last_pipe)
     {
       /* this is an empty sequence, insert a do-nothing LogPipe */
-      first_pipe = last_pipe = log_pipe_new();
+      first_pipe = last_pipe = log_pipe_new(self->cfg);
       g_ptr_array_add(self->initialized_pipes, first_pipe);
     }
   
@@ -824,16 +824,14 @@ cfg_tree_compile_junction(CfgTree *self,
 
           if (!is_first_branch && !fork_mpx)
             {
-              gchar buf[128];
-
               msg_error("Error compiling junction, source and non-source branches are mixed",
-                        evt_tag_str("location", log_expr_node_format_location(ep, buf, sizeof(buf))),
+                        log_expr_node_location_tag(ep),
                         NULL);
               goto error;
             }
           if (!fork_mpx)
             {
-              fork_mpx = log_multiplexer_new(0);
+              fork_mpx = log_multiplexer_new(0, self->cfg);
               g_ptr_array_add(self->initialized_pipes, &fork_mpx->super);
             }
           log_multiplexer_add_next_hop(fork_mpx, sub_pipe_head);
@@ -844,10 +842,8 @@ cfg_tree_compile_junction(CfgTree *self,
 
           if (fork_mpx)
             {
-              gchar buf[128];
-
               msg_error("Error compiling junction, source and non-source branches are mixed",
-                        evt_tag_str("location", log_expr_node_format_location(ep, buf, sizeof(buf))),
+                        log_expr_node_location_tag(ep),
                         NULL);
               goto error;
             }
@@ -857,7 +853,7 @@ cfg_tree_compile_junction(CfgTree *self,
         {
           if (!join_pipe)
             {
-              join_pipe = log_pipe_new();
+              join_pipe = log_pipe_new(self->cfg);
               g_ptr_array_add(self->initialized_pipes, join_pipe);
             }
           log_pipe_append(sub_pipe_tail, join_pipe);
@@ -892,15 +888,18 @@ cfg_tree_compile_node(CfgTree *self, LogExprNode *node,
 
   if (debug_flag)
     {
-      gchar buf[32];
+      gchar buf[128];
+      gchar compile_message[256];
 
       indent++;
-      fprintf(stderr, "%.*sCompiling %s %s [%s] at [%s]\n",
-              indent * 2, "                   ",
-              node->name ? : "#unnamed",
-              log_expr_node_get_layout_name(node->layout),
-              log_expr_node_get_content_name(node->content),
-              log_expr_node_format_location(node, buf, sizeof(buf)));
+      g_snprintf(compile_message, sizeof(compile_message),
+                 "%-*sCompiling %s %s [%s] at [%s]",
+                  indent * 2, "",
+                  node->name ? : "#unnamed",
+                  log_expr_node_get_layout_name(node->layout),
+                  log_expr_node_get_content_name(node->content),
+                  log_expr_node_format_location(node, buf, sizeof(buf)));
+      msg_send_formatted_message(EVT_PRI_DEBUG, compile_message);
     }
 
   switch (node->layout)
@@ -1071,7 +1070,9 @@ cfg_tree_start(CfgTree *self)
    */
   for (i = 0; i < self->initialized_pipes->len; i++)
     {
-      if (!log_pipe_init(g_ptr_array_index(self->initialized_pipes, i), self->cfg))
+      LogPipe *pipe = g_ptr_array_index(self->initialized_pipes, i);
+
+      if (!log_pipe_init(pipe))
         {
           msg_error("Error initializing message pipeline",
                     NULL);
