@@ -29,9 +29,11 @@
 #include "timeutils.h"
 #include "tags.h"
 #include "nvtable.h"
-#include "stats.h"
-#include "templates.h"
+#include "stats/stats-registry.h"
+#include "template/templates.h"
 #include "tls-support.h"
+#include "compat/string.h"
+#include "rcptid.h"
 
 #include <sys/types.h>
 #include <time.h>
@@ -121,6 +123,8 @@ TLS_BLOCK_START
   gint logmsg_cached_refs;
   /* number of cached acks by the current thread */
   gint logmsg_cached_acks;
+  /* abort flag in the current thread for acks */
+  gboolean logmsg_cached_abort;
 }
 TLS_BLOCK_END;
 
@@ -128,20 +132,23 @@ TLS_BLOCK_END;
 #define logmsg_cached_refs          __tls_deref(logmsg_cached_refs)
 #define logmsg_cached_acks          __tls_deref(logmsg_cached_acks)
 #define logmsg_cached_ack_needed    __tls_deref(logmsg_cached_ack_needed)
+#define logmsg_cached_abort         __tls_deref(logmsg_cached_abort)
 
-#define LOGMSG_REFCACHE_BIAS                  0x00004000 /* the BIAS we add to the ref counter in refcache_start */
-#define LOGMSG_REFCACHE_ACK_SHIFT                     16 /* number of bits to shift to get the ACK counter */
-#define LOGMSG_REFCACHE_ACK_MASK              0xFFFF0000 /* bit mask to extract the ACK counter */
+#define LOGMSG_REFCACHE_ABORT_SHIFT                   30 /* number of bits to shift to get the ABORT flag */
+#define LOGMSG_REFCACHE_ABORT_MASK            0x40000000 /* bit mask to extract the ABORT flag */
+#define LOGMSG_REFCACHE_ACK_SHIFT                     15 /* number of bits to shift to get the ACK counter */
+#define LOGMSG_REFCACHE_ACK_MASK              0x3FFF8000 /* bit mask to extract the ACK counter */
 #define LOGMSG_REFCACHE_REF_SHIFT                      0 /* number of bits to shift to get the REF counter */
-#define LOGMSG_REFCACHE_REF_MASK              0x0000FFFF /* bit mask to extract the ACK counter */
+#define LOGMSG_REFCACHE_REF_MASK              0x00007FFF /* bit mask to extract the ACK counter */
+#define LOGMSG_REFCACHE_BIAS                  0x00002000 /* the BIAS we add to the ref counter in refcache_start */
 
+#define LOGMSG_REFCACHE_REF_TO_VALUE(x)    (((x) << LOGMSG_REFCACHE_REF_SHIFT)   & LOGMSG_REFCACHE_REF_MASK)
+#define LOGMSG_REFCACHE_ACK_TO_VALUE(x)    (((x) << LOGMSG_REFCACHE_ACK_SHIFT)   & LOGMSG_REFCACHE_ACK_MASK)
+#define LOGMSG_REFCACHE_ABORT_TO_VALUE(x)  (((x) << LOGMSG_REFCACHE_ABORT_SHIFT) & LOGMSG_REFCACHE_ABORT_MASK)
 
-#define LOGMSG_REFCACHE_REF_TO_VALUE(x)    (((x) << LOGMSG_REFCACHE_REF_SHIFT) & LOGMSG_REFCACHE_REF_MASK)
-#define LOGMSG_REFCACHE_ACK_TO_VALUE(x)    (((x) << LOGMSG_REFCACHE_ACK_SHIFT) & LOGMSG_REFCACHE_ACK_MASK)
-
-#define LOGMSG_REFCACHE_VALUE_TO_REF(x)    (((x) & LOGMSG_REFCACHE_REF_MASK) >> LOGMSG_REFCACHE_REF_SHIFT)
-#define LOGMSG_REFCACHE_VALUE_TO_ACK(x)    (((x) & LOGMSG_REFCACHE_ACK_MASK) >> LOGMSG_REFCACHE_ACK_SHIFT)
-
+#define LOGMSG_REFCACHE_VALUE_TO_REF(x)    (((x) & LOGMSG_REFCACHE_REF_MASK)   >> LOGMSG_REFCACHE_REF_SHIFT)
+#define LOGMSG_REFCACHE_VALUE_TO_ACK(x)    (((x) & LOGMSG_REFCACHE_ACK_MASK)   >> LOGMSG_REFCACHE_ACK_SHIFT)
+#define LOGMSG_REFCACHE_VALUE_TO_ABORT(x)  (((x) & LOGMSG_REFCACHE_ABORT_MASK) >> LOGMSG_REFCACHE_ABORT_SHIFT)
 
 /**********************************************************************
  * LogMessage
@@ -379,7 +386,7 @@ log_msg_get_macro_value(LogMessage *self, gint id, gssize *value_len)
     }
   g_string_truncate(value, 0);
 
-  log_macro_expand(value, id, FALSE, NULL, LTZ_LOCAL, 0, NULL, self);
+  log_macro_expand_simple(value, id, self);
   if (value_len)
     *value_len = value->len;
   return value->str;
@@ -972,7 +979,7 @@ log_msg_init(LogMessage *self, GSockAddr *saddr)
   GTimeVal tv;
 
   /* ref is set to 1, ack is set to 0 */
-  self->ack_and_ref = LOGMSG_REFCACHE_REF_TO_VALUE(1);
+  self->ack_and_ref_and_abort = LOGMSG_REFCACHE_REF_TO_VALUE(1);
   cached_g_current_time(&tv);
   self->timestamps[LM_TS_RECVD].tv_sec = tv.tv_sec;
   self->timestamps[LM_TS_RECVD].tv_usec = tv.tv_usec;
@@ -985,6 +992,8 @@ log_msg_init(LogMessage *self, GSockAddr *saddr)
 
   self->original = NULL;
   self->flags |= LF_STATE_OWN_MASK;
+
+  self->rcptid = rcptid_generate_id();
 }
 
 void
@@ -997,13 +1006,22 @@ log_msg_clear(LogMessage *self)
 
   if (log_msg_chk_flag(self, LF_STATE_OWN_TAGS) && self->tags)
     {
-      memset(self->tags, 0, self->num_tags * sizeof(self->tags[0]));
+      gboolean inline_tags = self->num_tags == 0;
+
+      if (inline_tags)
+        self->tags = NULL;
+      else
+        memset(self->tags, 0, self->num_tags * sizeof(self->tags[0]));
     }
   else
     self->tags = NULL;
 
   self->num_matches = 0;
-  /* alloc_sdata remains */
+  if (!log_msg_chk_flag(self, LF_STATE_OWN_SDATA))
+    {
+      self->sdata = NULL;
+      self->alloc_sdata = 0;
+    }
   self->num_sdata = 0;
 
   if (log_msg_chk_flag(self, LF_STATE_OWN_SADDR) && self->saddr)
@@ -1088,13 +1106,13 @@ log_msg_new_empty(void)
 }
 
 void
-log_msg_clone_ack(LogMessage *msg, gpointer user_data)
+log_msg_clone_ack(LogMessage *msg, gpointer user_data, AckType ack_type)
 {
   LogPathOptions path_options = LOG_PATH_OPTIONS_INIT;
 
   g_assert(msg->original);
   path_options.ack_needed = TRUE;
-  log_msg_ack(msg->original, &path_options);
+  log_msg_ack(msg->original, &path_options, ack_type);
 }
 
 /*
@@ -1125,7 +1143,7 @@ log_msg_clone_cow(LogMessage *msg, const LogPathOptions *path_options)
 
   /* reference the original message */
   self->original = log_msg_ref(msg);
-  self->ack_and_ref = LOGMSG_REFCACHE_REF_TO_VALUE(1) + LOGMSG_REFCACHE_ACK_TO_VALUE(0);
+  self->ack_and_ref_and_abort = LOGMSG_REFCACHE_REF_TO_VALUE(1) + LOGMSG_REFCACHE_ACK_TO_VALUE(0) + LOGMSG_REFCACHE_ABORT_TO_VALUE(0);
   self->cur_node = 0;
   self->protect_cnt = 0;
 
@@ -1133,12 +1151,10 @@ log_msg_clone_cow(LogMessage *msg, const LogPathOptions *path_options)
   if (!path_options->ack_needed)
     {
       self->ack_func  = NULL;
-      self->ack_userdata = NULL;
     }
   else
     {
       self->ack_func = (LMAckFunc) log_msg_clone_ack;
-      self->ack_userdata = NULL;
     }
 
   self->flags &= ~LF_STATE_MASK;
@@ -1227,7 +1243,7 @@ log_msg_free(LogMessage *self)
 void
 log_msg_drop(LogMessage *msg, const LogPathOptions *path_options)
 {
-  log_msg_ack(msg, path_options);
+  log_msg_ack(msg, path_options, AT_PROCESSED);
   log_msg_unref(msg);
 }
 
@@ -1237,20 +1253,27 @@ log_msg_drop(LogMessage *msg, const LogPathOptions *path_options)
  * of this file about ref/ack handling is strongly recommended.
  ***************************************************************************************/
 
-/* Function to update the combined ACK and REF counter. */
+/* Function to update the combined ACK (with the abort flag) and REF counter. */
+static inline gint
+log_msg_update_ack_and_ref_and_abort(LogMessage *self, gint add_ref, gint add_ack, gint add_abort)
+{
+  gint old_value, new_value;
+  do
+    {
+      new_value = old_value = (volatile gint) self->ack_and_ref_and_abort;
+      new_value = (new_value & ~LOGMSG_REFCACHE_REF_MASK)   + LOGMSG_REFCACHE_REF_TO_VALUE(  (LOGMSG_REFCACHE_VALUE_TO_REF(old_value)   + add_ref));
+      new_value = (new_value & ~LOGMSG_REFCACHE_ACK_MASK)   + LOGMSG_REFCACHE_ACK_TO_VALUE(  (LOGMSG_REFCACHE_VALUE_TO_ACK(old_value)   + add_ack));
+      new_value = (new_value & ~LOGMSG_REFCACHE_ABORT_MASK) + LOGMSG_REFCACHE_ABORT_TO_VALUE((LOGMSG_REFCACHE_VALUE_TO_ABORT(old_value) | add_abort));
+    }
+  while (!g_atomic_int_compare_and_exchange(&self->ack_and_ref_and_abort, old_value, new_value));
+  return old_value;
+}
+
+/* Function to update the combined ACK (without abort) and REF counter. */
 static inline gint
 log_msg_update_ack_and_ref(LogMessage *self, gint add_ref, gint add_ack)
 {
-  gint old_value, new_value;
-
-  do
-    {
-      new_value = old_value = (volatile gint) self->ack_and_ref;
-      new_value = (new_value & ~LOGMSG_REFCACHE_REF_MASK) + LOGMSG_REFCACHE_REF_TO_VALUE((LOGMSG_REFCACHE_VALUE_TO_REF(old_value) + add_ref));
-      new_value = (new_value & ~LOGMSG_REFCACHE_ACK_MASK) + LOGMSG_REFCACHE_ACK_TO_VALUE((LOGMSG_REFCACHE_VALUE_TO_ACK(old_value) + add_ack));
-    }
-  while (!g_atomic_int_compare_and_exchange(&self->ack_and_ref, old_value, new_value));
-  return old_value;
+  return log_msg_update_ack_and_ref_and_abort(self, add_ref, add_ack, 0);
 }
 
 /**
@@ -1308,7 +1331,6 @@ log_msg_unref(LogMessage *self)
     }
 }
 
-
 /**
  * log_msg_add_ack:
  * @m: LogMessage instance
@@ -1336,12 +1358,13 @@ log_msg_add_ack(LogMessage *self, const LogPathOptions *path_options)
  * log_msg_ack:
  * @msg: LogMessage instance
  * @path_options: path specific options
+ * @acked: TRUE: positive ack, FALSE: negative ACK
  *
  * Indicate that the message was processed successfully and the sender can
  * queue further messages.
  **/
 void
-log_msg_ack(LogMessage *self, const LogPathOptions *path_options)
+log_msg_ack(LogMessage *self, const LogPathOptions *path_options, AckType ack_type)
 {
   gint old_value;
 
@@ -1353,13 +1376,17 @@ log_msg_ack(LogMessage *self, const LogPathOptions *path_options)
            * delayed until log_msg_refcache_stop() is called */
 
           logmsg_cached_acks--;
+          logmsg_cached_abort |= (ACKTYPE_TO_ABORTFLAG(ack_type) == AT_ABORTED ? TRUE : FALSE);
           return;
         }
 
-      old_value = log_msg_update_ack_and_ref(self, 0, -1);
+      old_value = log_msg_update_ack_and_ref_and_abort(self, 0, -1, ACKTYPE_TO_ABORTFLAG(ack_type));
       if (LOGMSG_REFCACHE_VALUE_TO_ACK(old_value) == 1)
         {
-          self->ack_func(self, self->ack_userdata);
+          if (ack_type == AT_ABORTED)
+            self->ack_func(self, AT_ABORTED);
+          else
+            self->ack_func(self, ABORTFLAG_TO_ACKTYPE(LOGMSG_REFCACHE_VALUE_TO_ABORT(old_value)));
         }
     }
 }
@@ -1378,7 +1405,7 @@ log_msg_break_ack(LogMessage *msg, const LogPathOptions *path_options, LogPathOp
 
   g_assert(!path_options->flow_control_requested);
 
-  log_msg_ack(msg, path_options);
+  log_msg_ack(msg, path_options, AT_PROCESSED);
 
   *local_options = *path_options;
   local_options->ack_needed = FALSE;
@@ -1412,10 +1439,12 @@ log_msg_refcache_start_producer(LogMessage *self)
 
   /* we don't need to be thread-safe here, as a producer has just created this message and no parallel access is yet possible */
 
-  self->ack_and_ref = (self->ack_and_ref & ~LOGMSG_REFCACHE_REF_MASK) + LOGMSG_REFCACHE_REF_TO_VALUE((LOGMSG_REFCACHE_VALUE_TO_REF(self->ack_and_ref) + LOGMSG_REFCACHE_BIAS));
-  self->ack_and_ref = (self->ack_and_ref & ~LOGMSG_REFCACHE_ACK_MASK) + LOGMSG_REFCACHE_ACK_TO_VALUE((LOGMSG_REFCACHE_VALUE_TO_ACK(self->ack_and_ref) + LOGMSG_REFCACHE_BIAS));
+  self->ack_and_ref_and_abort = (self->ack_and_ref_and_abort & ~LOGMSG_REFCACHE_REF_MASK) + LOGMSG_REFCACHE_REF_TO_VALUE((LOGMSG_REFCACHE_VALUE_TO_REF(self->ack_and_ref_and_abort) + LOGMSG_REFCACHE_BIAS));
+  self->ack_and_ref_and_abort = (self->ack_and_ref_and_abort & ~LOGMSG_REFCACHE_ACK_MASK) + LOGMSG_REFCACHE_ACK_TO_VALUE((LOGMSG_REFCACHE_VALUE_TO_ACK(self->ack_and_ref_and_abort) + LOGMSG_REFCACHE_BIAS));
+
   logmsg_cached_refs = -LOGMSG_REFCACHE_BIAS;
   logmsg_cached_acks = -LOGMSG_REFCACHE_BIAS;
+  logmsg_cached_abort = FALSE;
   logmsg_cached_ack_needed = TRUE;
 }
 
@@ -1440,9 +1469,9 @@ log_msg_refcache_start_consumer(LogMessage *self, const LogPathOptions *path_opt
   logmsg_cached_ack_needed = path_options->ack_needed;
   logmsg_cached_refs = 0;
   logmsg_cached_acks = 0;
+  logmsg_cached_abort = FALSE;
 }
 
-
 /*
  * Stop caching ref/unref/ack/add-ack operations in the current thread for
  * the message specified by the log_msg_refcache_start() function.
@@ -1454,6 +1483,7 @@ log_msg_refcache_stop(void)
 {
   gint old_value;
   gint current_cached_acks;
+  gboolean current_cached_abort;
 
   g_assert(logmsg_current != NULL);
 
@@ -1508,12 +1538,17 @@ log_msg_refcache_stop(void)
 
   current_cached_acks = logmsg_cached_acks;
   logmsg_cached_acks = 0;
-  old_value = log_msg_update_ack_and_ref(logmsg_current, 0, current_cached_acks);
+
+  current_cached_abort = logmsg_cached_abort;
+  logmsg_cached_abort = FALSE;
+
+  old_value = log_msg_update_ack_and_ref_and_abort(logmsg_current, 0, current_cached_acks, current_cached_abort);
 
   if ((LOGMSG_REFCACHE_VALUE_TO_ACK(old_value) == -current_cached_acks) && logmsg_cached_ack_needed)
     {
       /* 3) call the ack handler */
-      logmsg_current->ack_func(logmsg_current, logmsg_current->ack_userdata);
+      AckType ack_type_cummulated = ABORTFLAG_TO_ACKTYPE(LOGMSG_REFCACHE_VALUE_TO_ABORT(old_value));
+      logmsg_current->ack_func(logmsg_current, ack_type_cummulated);
 
       /* the ack callback may not change the ack counters, it already
        * dropped to zero atomically, changing that again is an error */
