@@ -26,8 +26,8 @@
 #include "logpipe.h"
 #include "messages.h"
 #include "serialize.h"
-#include "stats.h"
-#include "mainloop.h"
+#include "stats/stats-registry.h"
+#include "mainloop-worker.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -87,7 +87,7 @@ typedef struct _LogQueueFifo
   struct
   {
     struct iv_list_head items;
-    MainLoopIOWorkerFinishCallback cb;
+    WorkerBatchCallback cb;
     guint16 len;
     guint16 finish_cb_registered;
   } qoverflow_input[0];
@@ -109,11 +109,34 @@ log_queue_fifo_get_length(LogQueue *s)
   return self->qoverflow_wait_len + self->qoverflow_output_len;
 }
 
+gboolean
+log_queue_fifo_is_empty_racy(LogQueue *s)
+{
+  LogQueueFifo *self = (LogQueueFifo *) s;
+  gboolean has_message_in_queue = FALSE;
+  g_static_mutex_lock(&self->super.lock);
+  if (log_queue_fifo_get_length(s) > 0)
+    {
+      has_message_in_queue = TRUE;
+    }
+  else
+    {
+      gint i;
+      for (i = 0; i < log_queue_max_threads && !has_message_in_queue; i++)
+        {
+          has_message_in_queue |= self->qoverflow_input[i].finish_cb_registered;
+        }
+    }
+  g_static_mutex_unlock(&self->super.lock);
+  return !has_message_in_queue;
+}
+
 /* NOTE: this is inherently racy, can only be called if log processing is suspended (e.g. reload time) */
 static gboolean
 log_queue_fifo_keep_on_reload(LogQueue *s)
 {
-  return log_queue_fifo_get_length(s) > 0;
+  LogQueueFifo *self = (LogQueueFifo *) s;
+  return log_queue_fifo_get_length(s) > 0 || self->qbacklog_len > 0;
 }
 
 /* move items from the per-thread input queue to the lock-protected "wait" queue */
@@ -160,10 +183,11 @@ log_queue_fifo_move_input_unlocked(LogQueueFifo *self, gint thread_id)
           log_msg_free_queue_node(node);
           log_msg_drop(msg, &path_options);
         }
-      msg_debug("Destination queue full, dropping messages",
+      msg_error("Destination queue full, dropping messages",
                 evt_tag_int("queue_len", queue_len),
                 evt_tag_int("log_fifo_size", self->qoverflow_size),
                 evt_tag_int("count", n),
+                evt_tag_str("persist_name", self->super.persist_name),
                 NULL);
     }
   stats_counter_add(self->super.stored_messages, self->qoverflow_input[thread_id].len);
@@ -183,7 +207,7 @@ log_queue_fifo_move_input(gpointer user_data)
   LogQueueFifo *self = (LogQueueFifo *) user_data;
   gint thread_id;
 
-  thread_id = main_loop_io_worker_thread_id();
+  thread_id = main_loop_worker_get_thread_id();
 
   g_assert(thread_id >= 0);
 
@@ -213,7 +237,7 @@ log_queue_fifo_push_tail(LogQueue *s, LogMessage *msg, const LogPathOptions *pat
   gint thread_id;
   LogMessageQueueNode *node;
 
-  thread_id = main_loop_io_worker_thread_id();
+  thread_id = main_loop_worker_get_thread_id();
 
   g_assert(thread_id < 0 || log_queue_max_threads > thread_id);
 
@@ -236,7 +260,7 @@ log_queue_fifo_push_tail(LogQueue *s, LogMessage *msg, const LogPathOptions *pat
            * callback to make sure it gets moved to the wait_queue if the
            * input thread finishes */
 
-          main_loop_io_worker_register_finish_callback(&self->qoverflow_input[thread_id].cb);
+          main_loop_worker_register_batch_callback(&self->qoverflow_input[thread_id].cb);
           self->qoverflow_input[thread_id].finish_cb_registered = TRUE;
         }
 
@@ -276,6 +300,7 @@ log_queue_fifo_push_tail(LogQueue *s, LogMessage *msg, const LogPathOptions *pat
       msg_debug("Destination queue full, dropping message",
                 evt_tag_int("queue_len", log_queue_fifo_get_length(&self->super)),
                 evt_tag_int("log_fifo_size", self->qoverflow_size),
+                evt_tag_str("persist_name", self->super.persist_name),
                 NULL);
     }
   return;
@@ -311,17 +336,13 @@ log_queue_fifo_push_head(LogQueue *s, LogMessage *msg, const LogPathOptions *pat
  *
  * NOTE: this returns a reference which the caller must take care to free.
  */
-static gboolean
-log_queue_fifo_pop_head(LogQueue *s, LogMessage **msg, LogPathOptions *path_options, gboolean push_to_backlog, gboolean ignore_throttle)
+static LogMessage *
+log_queue_fifo_pop_head(LogQueue *s, LogPathOptions *path_options)
 {
   LogQueueFifo *self = (LogQueueFifo *) s;
   LogMessageQueueNode *node;
+  LogMessage *msg = NULL;
 
-  if (!ignore_throttle && self->super.throttle && self->super.throttle_buckets == 0)
-    {
-      return FALSE;
-    }
-    
   if (self->qoverflow_output_len == 0)
     {
       /* slow path, output queue is empty, get some elements from the wait queue */
@@ -336,10 +357,10 @@ log_queue_fifo_pop_head(LogQueue *s, LogMessage **msg, LogPathOptions *path_opti
     {
       node = iv_list_entry(self->qoverflow_output.next, LogMessageQueueNode, list);
 
-      *msg = node->msg;
+      msg = node->msg;
       path_options->ack_needed = node->ack_needed;
       self->qoverflow_output_len--;
-      if (!push_to_backlog)
+      if (!self->super.use_backlog)
         {
           iv_list_del(&node->list);
           log_msg_free_queue_node(node);
@@ -359,55 +380,49 @@ log_queue_fifo_pop_head(LogQueue *s, LogMessage **msg, LogPathOptions *path_opti
        * the high watermark is reached). Also, they are unlocked, so
        * no way to touch them safely.
        */
-      return FALSE;
+      return NULL;
     }
   stats_counter_dec(self->super.stored_messages);
 
-  if (push_to_backlog)
+  if (self->super.use_backlog)
     {
-      log_msg_ref(*msg);
+      log_msg_ref(msg);
       iv_list_add_tail(&node->list, &self->qbacklog);
       self->qbacklog_len++;
     }
-  if (!ignore_throttle && self->super.throttle_buckets > 0)
-    {
-      self->super.throttle_buckets--;
-    }
 
-  return TRUE;
+  return msg;
 }
 
 /*
  * Can only run from the output thread.
  */
 static void
-log_queue_fifo_ack_backlog(LogQueue *s, gint n)
+log_queue_fifo_ack_backlog(LogQueue *s, gint rewind_count)
 {
   LogQueueFifo *self = (LogQueueFifo *) s;
   LogMessage *msg;
   LogPathOptions path_options = LOG_PATH_OPTIONS_INIT;
-  gint i;
+  gint pos;
 
-  for (i = 0; i < n && self->qbacklog_len > 0; i++)
+  for (pos = 0; pos < rewind_count && self->qbacklog_len > 0; pos++)
     {
       LogMessageQueueNode *node;
-
       node = iv_list_entry(self->qbacklog.next, LogMessageQueueNode, list);
       msg = node->msg;
-      path_options.ack_needed = node->ack_needed;
 
       iv_list_del(&node->list);
-      log_msg_free_queue_node(node);
       self->qbacklog_len--;
-
-      log_msg_ack(msg, &path_options);
+      path_options.ack_needed = node->ack_needed;
+      log_msg_ack(msg, &path_options, AT_PROCESSED);
+      log_msg_free_queue_node(node);
       log_msg_unref(msg);
     }
 }
 
 
 /*
- * log_queue_rewind_backlog:
+ * log_queue_rewind_backlog_all:
  *
  * Move items on our backlog back to our qoverflow queue. Please note that this
  * function does not really care about qoverflow size, it has to put the backlog
@@ -417,7 +432,7 @@ log_queue_fifo_ack_backlog(LogQueue *s, gint n)
  * NOTE: this is assumed to be called from the output thread.
  */
 static void
-log_queue_fifo_rewind_backlog(LogQueue *s)
+log_queue_fifo_rewind_backlog_all(LogQueue *s)
 {
   LogQueueFifo *self = (LogQueueFifo *) s;
 
@@ -427,6 +442,32 @@ log_queue_fifo_rewind_backlog(LogQueue *s)
   self->qbacklog_len = 0;
 }
 
+static void
+log_queue_fifo_rewind_backlog(LogQueue *s, guint rewind_count)
+{
+  LogQueueFifo *self = (LogQueueFifo *) s;
+  guint pos;
+
+  if (rewind_count > self->qbacklog_len)
+    rewind_count = self->qbacklog_len;
+
+  for (pos = 0; pos < rewind_count; pos++)
+    {
+      LogMessageQueueNode *node = iv_list_entry(self->qbacklog.prev, LogMessageQueueNode, list);
+      /*
+       * Because the message go to the backlog only in case of pop_head
+       * and pop_head add ack and ref when it pushes the message into the backlog
+       * The rewind must decrease the ack and ref too
+       */
+      iv_list_del_init(&node->list);
+      iv_list_add(&node->list, &self->qoverflow_output);
+
+      self->qbacklog_len--;
+      self->qoverflow_output_len++;
+      stats_counter_inc(self->super.stored_messages);
+    }
+}
+
 static void
 log_queue_fifo_free_queue(struct iv_list_head *q)
 {
@@ -442,7 +483,7 @@ log_queue_fifo_free_queue(struct iv_list_head *q)
       path_options.ack_needed = node->ack_needed;
       msg = node->msg;
       log_msg_free_queue_node(node);
-      log_msg_ack(msg, &path_options);
+      log_msg_ack(msg, &path_options, AT_ABORTED);
       log_msg_unref(msg);
     }
 }
@@ -471,20 +512,23 @@ log_queue_fifo_new(gint qoverflow_size, const gchar *persist_name)
   self = g_malloc0(sizeof(LogQueueFifo) + log_queue_max_threads * sizeof(self->qoverflow_input[0]));
 
   log_queue_init_instance(&self->super, persist_name);
+  self->super.use_backlog = FALSE;
   self->super.get_length = log_queue_fifo_get_length;
+  self->super.is_empty_racy = log_queue_fifo_is_empty_racy;
   self->super.keep_on_reload = log_queue_fifo_keep_on_reload;
   self->super.push_tail = log_queue_fifo_push_tail;
   self->super.push_head = log_queue_fifo_push_head;
   self->super.pop_head = log_queue_fifo_pop_head;
   self->super.ack_backlog = log_queue_fifo_ack_backlog;
   self->super.rewind_backlog = log_queue_fifo_rewind_backlog;
+  self->super.rewind_backlog_all = log_queue_fifo_rewind_backlog_all;
 
   self->super.free_fn = log_queue_fifo_free;
   
   for (i = 0; i < log_queue_max_threads; i++)
     {
       INIT_IV_LIST_HEAD(&self->qoverflow_input[i].items);
-      main_loop_io_worker_finish_callback_init(&self->qoverflow_input[i].cb);
+      worker_batch_callback_init(&self->qoverflow_input[i].cb);
       self->qoverflow_input[i].cb.user_data = self;
       self->qoverflow_input[i].cb.func = log_queue_fifo_move_input;
     }
