@@ -24,9 +24,13 @@
   
 #include "logwriter.h"
 #include "messages.h"
-#include "stats.h"
+#include "stats/stats-registry.h"
+#include "hostname.h"
+#include "host-resolve.h"
 #include "misc.h"
 #include "mainloop.h"
+#include "mainloop-io-worker.h"
+#include "mainloop-call.h"
 #include "ml-batched-timer.h"
 #include "str-format.h"
 
@@ -47,9 +51,7 @@ typedef enum
   /* business as usual, flush when the buffer is full */
   LW_FLUSH_NORMAL,
   /* flush the buffer immediately please */
-  LW_FLUSH_BUFFER,
-  /* pull off any queued items, at maximum speed, even ignoring throttle, and flush the buffer too */
-  LW_FLUSH_QUEUE,
+  LW_FLUSH_FORCE,
 } LogWriterFlushMode;
 
 struct _LogWriter
@@ -84,7 +86,7 @@ struct _LogWriter
   gboolean work_result;
   gint pollable_state;
   LogProtoClient *proto, *pending_proto;
-  gboolean watches_running:1, suspended:1, working:1, flush_waiting_for_timeout:1;
+  gboolean watches_running:1, suspended:1, working:1, waiting_for_throttle:1;
   gboolean pending_proto_present;
   GCond *pending_proto_cond;
   GStaticMutex pending_proto_lock;
@@ -119,6 +121,60 @@ static void log_writer_start_watches(LogWriter *self);
 static void log_writer_stop_watches(LogWriter *self);
 static void log_writer_update_watches(LogWriter *self);
 static void log_writer_suspend(LogWriter *self);
+static void log_writer_free_proto(LogWriter *self);
+static void log_writer_set_proto(LogWriter *self, LogProtoClient *proto);
+static void log_writer_set_pending_proto(LogWriter *self, LogProtoClient *proto, gboolean present);
+
+
+static void
+log_writer_msg_ack(gint num_msg_acked, gpointer user_data)
+{
+  LogWriter *self = (LogWriter *)user_data;
+  log_queue_ack_backlog(self->queue, num_msg_acked);
+}
+
+static void
+log_writer_msg_rewind(gpointer user_data)
+{
+  LogWriter *self = (LogWriter *)user_data;
+  log_queue_rewind_backlog_all(self->queue);
+}
+
+void
+log_writer_set_flags(LogWriter *self, guint32 flags)
+{
+  g_assert((self->super.flags & PIF_INITIALIZED) == 0);
+  self->flags = flags;
+}
+
+guint32
+log_writer_get_flags(LogWriter *self)
+{
+  return self->flags;
+}
+
+/* returns a reference */
+LogQueue *
+log_writer_get_queue(LogWriter *s)
+{
+  LogWriter *self = (LogWriter *) s;
+
+  return log_queue_ref(self->queue);
+}
+
+/* consumes the reference */
+void
+log_writer_set_queue(LogWriter *s, LogQueue *queue)
+{
+  LogWriter *self = (LogWriter *)s;
+
+  if (self->queue)
+    log_queue_unref(self->queue);
+
+  self->queue = queue;
+  if (self->queue)
+    log_queue_set_use_backlog(self->queue, TRUE);
+}
 
 static void
 log_writer_work_perform(gpointer s)
@@ -126,7 +182,7 @@ log_writer_work_perform(gpointer s)
   LogWriter *self = (LogWriter *) s;
 
   g_assert((self->super.flags & PIF_INITIALIZED) != 0);
-  self->work_result = log_writer_flush(self, self->flush_waiting_for_timeout ? LW_FLUSH_BUFFER : LW_FLUSH_NORMAL);
+  self->work_result = log_writer_flush(self, LW_FLUSH_NORMAL);
 }
 
 static void
@@ -135,7 +191,7 @@ log_writer_work_finished(gpointer s)
   LogWriter *self = (LogWriter *) s;
 
   main_loop_assert_main_thread();
-  self->flush_waiting_for_timeout = FALSE;
+  self->waiting_for_throttle = FALSE;
 
   if (self->pending_proto_present)
     {
@@ -145,12 +201,10 @@ log_writer_work_finished(gpointer s)
        * non-main thread. */
 
       g_static_mutex_lock(&self->pending_proto_lock);
-      if (self->proto)
-        log_proto_client_free(self->proto);
+      log_writer_free_proto(self);
 
-      self->proto = self->pending_proto;
-      self->pending_proto = NULL;
-      self->pending_proto_present = FALSE;
+      log_writer_set_proto(self, self->pending_proto);
+      log_writer_set_pending_proto(self, NULL, FALSE);
 
       g_cond_signal(self->pending_proto_cond);
       g_static_mutex_unlock(&self->pending_proto_lock);
@@ -204,7 +258,7 @@ log_writer_io_flush_output(gpointer s)
        * conditions of any kind.
        */
 
-      if (!main_loop_io_worker_job_quit())
+      if (!main_loop_worker_job_quit())
         {
           log_writer_work_perform(s);
           log_writer_work_finished(s);
@@ -380,7 +434,7 @@ log_writer_update_watches(LogWriter *self)
   /* NOTE: we either start the suspend_timer or enable the fd_watch. The two MUST not happen at the same time. */
 
   if (log_proto_client_prepare(self->proto, &fd, &cond) ||
-      self->flush_waiting_for_timeout ||
+      self->waiting_for_throttle ||
       log_queue_check_items(self->queue, &timeout_msec,
                             (LogQueuePushNotifyFunc) log_writer_schedule_update_watches, self, NULL))
     {
@@ -392,7 +446,7 @@ log_writer_update_watches(LogWriter *self)
       /* few elements are available, but less than flush_lines, we need to start a timer to initiate a flush */
 
       log_writer_update_fd_callbacks(self, 0);
-      self->flush_waiting_for_timeout = TRUE;
+      self->waiting_for_throttle = TRUE;
       log_writer_arm_suspend_timer(self, (void (*)(void *)) log_writer_update_watches, timeout_msec);
     }
   else
@@ -530,7 +584,7 @@ log_writer_emit_suppress_summary(LogWriter *self)
   len = g_snprintf(buf, sizeof(buf), "Last message '%.20s' repeated %d times, suppressed by syslog-ng on %s",
                    log_msg_get_value(self->last_msg, LM_V_MESSAGE, NULL),
                    self->last_msg_count,
-                   get_local_hostname(NULL));
+                   get_local_hostname_fqdn());
   log_msg_set_value(m, LM_V_MESSAGE, buf, len);
 
   path_options.ack_needed = FALSE;
@@ -620,17 +674,17 @@ log_writer_mark_timeout(void *cookie)
 {
   LogWriter *self = (LogWriter *)cookie;
   LogPathOptions path_options = LOG_PATH_OPTIONS_INIT;
-  gchar hostname[256];
-  gsize hostname_len = sizeof(hostname);
+  const gchar *hostname;
+  gsize hostname_len;
   LogMessage *msg;
 
   main_loop_assert_main_thread();
 
   msg = log_msg_new_mark();
   /* timeout: there was no new message on the writer or it is in periodical mode */
-  resolve_sockaddr(hostname, &hostname_len, msg->saddr, self->options->use_dns, self->options->use_fqdn, self->options->use_dns_cache, self->options->normalize_hostnames);
+  hostname = resolve_sockaddr_to_hostname(&hostname_len, msg->saddr, &self->options->host_resolve_options);
 
-  log_msg_set_value(msg, LM_V_HOST, hostname, strlen(hostname));
+  log_msg_set_value(msg, LM_V_HOST, hostname, hostname_len);
 
   /* set the current time in the message stamp */
   msg->timestamps[LM_TS_STAMP] = msg->timestamps[LM_TS_RECVD];
@@ -795,15 +849,6 @@ log_writer_format_log(LogWriter *self, LogMessage *lm, GString *result)
       log_writer_append_value(result, lm, LM_V_PID, TRUE, TRUE);
       log_writer_append_value(result, lm, LM_V_MSGID, TRUE, TRUE);
 
-#if 0
-      if (lm->flags & LF_LOCAL)
-        {
-          gchar sequence_id[16];
-          
-          g_snprintf(sequence_id, sizeof(sequence_id), "%d", seq_num);
-          log_msg_update_sdata(lm, "meta", "sequenceId", sequence_id);
-        }
-#endif
       len = result->len;
       log_msg_append_format_sdata(lm, result, seq_num);
       if (len == result->len)
@@ -940,7 +985,7 @@ static void
 log_writer_broken(LogWriter *self, gint notify_code)
 {
   log_writer_stop_watches(self);
-  log_pipe_notify(self->control, &self->super, notify_code, self);
+  log_pipe_notify(self->control, notify_code, self);
 }
 
 static void
@@ -961,25 +1006,121 @@ log_writer_realloc_line_buffer(LogWriter *self)
  * In threaded mode, this function is invoked as part of the "output" task
  * (in essence, this is the function that performs the output task).
  *
+ */
+
+static gboolean
+log_writer_flush_finalize(LogWriter *self)
+{
+  LogProtoStatus status = log_proto_client_flush(self->proto);
+
+  if (status != LPS_SUCCESS)
+    return FALSE;
+
+  return TRUE;
+}
+
+gboolean
+log_writer_write_message(LogWriter *self, LogMessage *msg, LogPathOptions *path_options, gboolean *write_error)
+{
+  gboolean consumed = FALSE;
+
+  *write_error = FALSE;
+
+  log_msg_refcache_start_consumer(msg, path_options);
+  msg_set_context(msg);
+
+  log_writer_format_log(self, msg, self->line_buffer);
+
+  if (!(msg->flags & LF_INTERNAL))
+    {
+      msg_debug("Outgoing message",
+            evt_tag_str("message",self->line_buffer->str),
+            NULL);
+    }
+
+  if (self->line_buffer->len)
+    {
+      LogProtoStatus status = log_proto_client_post(self->proto, (guchar *)self->line_buffer->str, self->line_buffer->len, &consumed);
+
+      if (consumed)
+        log_writer_realloc_line_buffer(self);
+
+      if (status == LPS_ERROR)
+        {
+          if ((self->options->options & LWO_IGNORE_ERRORS) != 0)
+            {
+              if (!consumed)
+                {
+                  g_free(self->line_buffer->str);
+                  log_writer_realloc_line_buffer(self);
+                  consumed = TRUE;
+                }
+            }
+          else
+            {
+              *write_error = TRUE;
+              consumed = FALSE;
+            }
+        }
+    }
+  else
+    {
+      msg_debug("Error posting log message as template() output resulted in an empty string, skipping message",
+                NULL);
+      consumed = TRUE;
+    }
+
+  if (consumed)
+    {
+      if (msg->flags & LF_LOCAL)
+        step_sequence_number(&self->seq_num);
+
+      log_msg_unref(msg);
+      msg_set_context(NULL);
+      log_msg_refcache_stop();
+
+      return TRUE;
+  }
+  else
+    {
+      msg_debug("Can't send the message rewind backlog",
+                evt_tag_str("message",self->line_buffer->str),
+                NULL);
+
+      log_queue_rewind_backlog(self->queue, 1);
+
+      log_msg_unref(msg);
+      msg_set_context(NULL);
+      log_msg_refcache_stop();
+
+      return FALSE;
+    }
+}
+
+static inline LogMessage *
+log_writer_queue_pop_message(LogWriter *self, LogPathOptions *path_options, gboolean force_flush)
+{
+  if (force_flush)
+    return log_queue_pop_head_ignore_throttle(self->queue, path_options);
+  else
+    return log_queue_pop_head(self->queue, path_options);
+}
+
+/*
  * @flush_mode specifies how hard LogWriter is trying to send messages to
  * the actual destination:
  *
  *
  * LW_FLUSH_NORMAL    - business as usual, flush when the buffer is full
- * LW_FLUSH_BUFFER    - flush the buffer immediately please
- * LW_FLUSH_QUEUE     - pull off any queued items, at maximum speed, even
- *                      ignoring throttle, and flush the buffer too
+ * LW_FLUSH_FORCE     - flush the buffer immediately please
  *
  */
 gboolean
 log_writer_flush(LogWriter *self, LogWriterFlushMode flush_mode)
 {
-  LogProtoClient *proto = self->proto;
-  gint count = 0;
-  gboolean ignore_throttle = (flush_mode >= LW_FLUSH_QUEUE);
-  LogProtoStatus status = LPS_SUCCESS;
-  
-  if (!proto)
+  gboolean write_error = FALSE;
+
+  if (!self->proto)
     return FALSE;
 
   /* NOTE: in case we're reloading or exiting we flush all queued items as
@@ -987,76 +1128,22 @@ log_writer_flush(LogWriter *self, LogWriterFlushMode flush_mode)
    * infinite loop, since the reader will cease to produce new messages when
    * main_loop_io_worker_job_quit() is set. */
 
-  while (status == LPS_SUCCESS && (!main_loop_io_worker_job_quit() || flush_mode >= LW_FLUSH_QUEUE))
+  while ((!main_loop_worker_job_quit() || flush_mode == LW_FLUSH_FORCE) && !write_error)
     {
-      LogMessage *lm;
       LogPathOptions path_options = LOG_PATH_OPTIONS_INIT;
-      gboolean consumed = FALSE;
-      
-      if (!log_queue_pop_head(self->queue, &lm, &path_options, FALSE, ignore_throttle))
-        {
-          /* no more items are available */
-          break;
-        }
+      LogMessage *msg = log_writer_queue_pop_message(self, &path_options, flush_mode == LW_FLUSH_FORCE);
 
-      log_msg_refcache_start_consumer(lm, &path_options);
-      msg_set_context(lm);
+      if (!msg)
+        break;
 
-      log_writer_format_log(self, lm, self->line_buffer);
-      
-      if (self->line_buffer->len)
-        {
-          status = log_proto_client_post(proto, (guchar *) self->line_buffer->str, self->line_buffer->len, &consumed);
-
-          if (consumed)
-            log_writer_realloc_line_buffer(self);
-
-          if (status == LPS_ERROR)
-            {
-              if ((self->options->options & LWO_IGNORE_ERRORS) != 0)
-                {
-                  consumed = TRUE;
-                  status = LPS_SUCCESS;
-                }
-            }
-        }
-      else
-        {
-          msg_debug("Error posting log message as template() output resulted in an empty string, skipping message",
-                    NULL);
-          consumed = TRUE;
-        }
-      if (consumed)
-        {
-          if (lm->flags & LF_LOCAL)
-            step_sequence_number(&self->seq_num);
-          log_msg_ack(lm, &path_options);
-          log_msg_unref(lm);
-        }
-      else
-        {
-          /* push back to the queue */
-          log_queue_push_head(self->queue, lm, &path_options);
-          msg_set_context(NULL);
-          log_msg_refcache_stop();
-          break;
-        }
-
-      msg_set_context(NULL);
-      log_msg_refcache_stop();
-      count++;
+      if (!log_writer_write_message(self, msg, &path_options, &write_error))
+        break;
     }
 
-  if (status != LPS_SUCCESS)
+  if (write_error)
     return FALSE;
 
-  if (flush_mode >= LW_FLUSH_BUFFER || count == 0)
-    {
-      if (log_proto_client_flush(proto) == LPS_ERROR)
-        return FALSE;
-    }
-
-  return TRUE;
+  return log_writer_flush_finalize(self);
 }
 
 static void
@@ -1099,7 +1186,10 @@ log_writer_init(LogPipe *s)
 {
   LogWriter *self = (LogWriter *) s;
 
-  g_assert(self->queue != NULL);
+  if (self->queue == NULL)
+    {
+      return FALSE;
+    }
   iv_event_register(&self->queue_filled);
 
   if ((self->options->options & LWO_NO_STATS) == 0 && !self->dropped_messages)
@@ -1119,8 +1209,8 @@ log_writer_init(LogPipe *s)
       LogProtoClient *proto;
 
       proto = self->proto;
-      self->proto = NULL;
-      log_writer_reopen(&self->super, proto);
+      log_writer_set_proto(self, NULL);
+      log_writer_reopen(self, proto);
     }
 
   if (self->options->mark_mode == MM_PERIODICAL)
@@ -1142,7 +1232,7 @@ log_writer_deinit(LogPipe *s)
   main_loop_assert_main_thread();
 
   log_queue_reset_parallel_push(self->queue);
-  log_writer_flush(self, LW_FLUSH_QUEUE);
+  log_writer_flush(self, LW_FLUSH_FORCE);
   /* FIXME: by the time we arrive here, it must be guaranteed that no
    * _queue() call is running in a different thread, otherwise we'd need
    * some kind of locking. */
@@ -1169,8 +1259,7 @@ log_writer_free(LogPipe *s)
 {
   LogWriter *self = (LogWriter *) s;
 
-  if (self->proto)
-    log_proto_client_free(self->proto);
+  log_writer_free_proto(self);
 
   if (self->line_buffer)
     g_string_free(self->line_buffer, TRUE);
@@ -1193,7 +1282,7 @@ log_writer_free(LogPipe *s)
 gboolean
 log_writer_has_pending_writes(LogWriter *self)
 {
-  return log_queue_get_length(self->queue) > 0 || !self->watches_running;
+  return !log_queue_is_empty_racy(self->queue) || !self->watches_running;
 }
 
 gboolean
@@ -1202,6 +1291,39 @@ log_writer_opened(LogWriter *self)
   return self->proto != NULL;
 }
 
+
+static void
+log_writer_free_proto(LogWriter *self)
+{
+  if (self->proto)
+    log_proto_client_free(self->proto);
+
+  self->proto = NULL;
+}
+
+static void
+log_writer_set_proto(LogWriter *self, LogProtoClient *proto)
+{
+  self->proto = proto;
+
+  if (proto)
+    {
+      LogProtoClientFlowControlFuncs flow_control_funcs;
+      flow_control_funcs.ack_callback = log_writer_msg_ack;
+      flow_control_funcs.rewind_callback = log_writer_msg_rewind;
+      flow_control_funcs.user_data = self;
+
+      log_proto_client_set_client_flow_control(self->proto, &flow_control_funcs);
+    }
+}
+
+static void
+log_writer_set_pending_proto(LogWriter *self, LogProtoClient *proto, gboolean present)
+{
+  self->pending_proto = proto;
+  self->pending_proto_present = present;
+}
+
 /* run in the main thread in reaction to a log_writer_reopen to change
  * the destination LogProtoClient instance. It needs to be ran in the main
  * thread as it reregisters the watches associated with the main
@@ -1217,18 +1339,15 @@ log_writer_reopen_deferred(gpointer s)
 
   if (self->io_job.working)
     {
-      /* NOTE: proto can be NULL */
-      self->pending_proto = proto;
-      self->pending_proto_present = TRUE;
+      /* NOTE: proto can be NULL but it is present... */
+      log_writer_set_pending_proto(self, proto, TRUE);
       return;
     }
 
   log_writer_stop_watches(self);
 
-  if (self->proto)
-    log_proto_client_free(self->proto);
-
-  self->proto = proto;
+  log_writer_free_proto(self);
+  log_writer_set_proto(self, proto);
 
   if (proto)
     log_writer_start_watches(self);
@@ -1261,7 +1380,7 @@ log_writer_reopen_deferred(gpointer s)
  * risky approach.
  */
 void
-log_writer_reopen(LogPipe *s, LogProtoClient *proto)
+log_writer_reopen(LogWriter *s, LogProtoClient *proto)
 {
   LogWriter *self = (LogWriter *) s;
   gpointer args[] = { s, proto };
@@ -1297,12 +1416,12 @@ log_writer_set_options(LogWriter *self, LogPipe *control, LogWriterOptions *opti
   self->stats_instance = stats_instance ? g_strdup(stats_instance) : NULL;
 }
 
-LogPipe *
-log_writer_new(guint32 flags)
+LogWriter *
+log_writer_new(guint32 flags, GlobalConfig *cfg)
 {
   LogWriter *self = g_new0(LogWriter, 1);
   
-  log_pipe_init_instance(&self->super);
+  log_pipe_init_instance(&self->super, cfg);
   self->super.init = log_writer_init;
   self->super.deinit = log_writer_deinit;
   self->super.queue = log_writer_queue;
@@ -1317,27 +1436,7 @@ log_writer_new(guint32 flags)
   g_static_mutex_init(&self->pending_proto_lock);
   self->pending_proto_cond = g_cond_new();
 
-  return &self->super;
-}
-
-/* returns a reference */
-LogQueue *
-log_writer_get_queue(LogPipe *s)
-{
-  LogWriter *self = (LogWriter *) s;
-
-  return log_queue_ref(self->queue);
-}
-
-/* consumes the reference */
-void
-log_writer_set_queue(LogPipe *s, LogQueue *queue)
-{
-  LogWriter *self = (LogWriter *)s;
-
-  if (self->queue)
-    log_queue_unref(self->queue);
-  self->queue = queue;
+  return self;
 }
 
 void 
@@ -1352,6 +1451,7 @@ log_writer_options_defaults(LogWriterOptions *options)
   options->padding = 0;
   options->mark_mode = MM_GLOBAL;
   options->mark_freq = -1;
+  host_resolve_options_defaults(&options->host_resolve_options);
 }
 
 void 
@@ -1404,6 +1504,7 @@ log_writer_options_init(LogWriterOptions *options, GlobalConfig *cfg, guint32 op
     return;
 
   log_template_options_init(&options->template_options, cfg);
+  host_resolve_options_init(&options->host_resolve_options, cfg);
   log_proto_client_options_init(&options->proto_options.super, cfg);
   options->options |= option_flags;
     
@@ -1431,10 +1532,6 @@ log_writer_options_init(LogWriterOptions *options, GlobalConfig *cfg, guint32 op
       options->mark_freq = cfg->mark_freq;
     }
 
-  options->use_dns = cfg->use_dns;
-  options->use_fqdn = cfg->use_fqdn;
-  options->use_dns_cache = cfg->use_dns_cache;
-  options->normalize_hostnames = cfg->normalize_hostnames;
   options->initialized = TRUE;
 }
 
@@ -1442,6 +1539,7 @@ void
 log_writer_options_destroy(LogWriterOptions *options)
 {
   log_template_options_destroy(&options->template_options);
+  host_resolve_options_destroy(&options->host_resolve_options);
   log_proto_client_options_destroy(&options->proto_options.super);
   log_template_unref(options->template);
   log_template_unref(options->file_template);
