@@ -143,10 +143,6 @@ nv_registry_free(NVRegistry *self)
 
 /* clonable LogMessage support with shared data pointers */
 
-#define NV_TABLE_DYNVALUE_HANDLE(x) ((x).handle)
-#define NV_TABLE_DYNVALUE_OFS(x)    ((x).ofs)
-
-
 static inline gchar *
 nv_table_get_bottom(NVTable *self)
 {
@@ -160,20 +156,6 @@ nv_table_get_ofs_table_top(NVTable *self)
                                self->num_dyn_entries * sizeof(NVDynValue)];
 }
 
-static inline NVEntry *
-nv_table_get_entry_at_ofs(NVTable *self, guint32 ofs)
-{
-  if (!ofs)
-    return NULL;
-  return (NVEntry *) (nv_table_get_top(self) - ofs);
-}
-
-static inline NVDynValue *
-nv_table_get_dyn_entries(NVTable *self)
-{
-  return (NVDynValue *) &self->static_entries[self->num_static_entries];
-}
-
 static inline gboolean
 nv_table_alloc_check(NVTable *self, gsize alloc_size)
 {
@@ -447,7 +429,7 @@ nv_table_add_value(NVTable *self, NVHandle handle, const gchar *name, gsize name
       return FALSE;
     }
 
-  ofs = (nv_table_get_top(self) - (gchar *) entry);
+  ofs = nv_table_get_dyn_value_offset_from_nventry(self, entry);
   entry->vdirect.value_len = value_len;
   if (handle >= self->num_static_entries)
     {
@@ -474,7 +456,7 @@ nv_table_add_value_indirect(NVTable *self, NVHandle handle, const gchar *name, g
   if (new_entry)
     *new_entry = FALSE;
   ref_entry = nv_table_get_entry(self, ref_handle, &dyn_slot);
-  if (ref_entry && ref_entry->indirect)
+  if ((ref_entry && ref_entry->indirect) || handle == ref_handle)
     {
       const gchar *ref_value;
       gssize ref_length;
@@ -482,7 +464,7 @@ nv_table_add_value_indirect(NVTable *self, NVHandle handle, const gchar *name, g
       /* NOTE: uh-oh, the to-be-referenced value is already an indirect
        * reference, this is not supported, copy the stuff */
 
-      ref_value = nv_table_resolve_indirect(self, ref_entry, &ref_length);
+      ref_value = nv_table_resolve_entry(self, ref_entry, &ref_length);
 
       if (rofs > ref_length)
         {
@@ -548,7 +530,7 @@ nv_table_add_value_indirect(NVTable *self, NVHandle handle, const gchar *name, g
       return FALSE;
     }
 
-  ofs = (nv_table_get_top(self) - (gchar *) entry);
+  ofs = nv_table_get_dyn_value_offset_from_nventry(self, entry);
   entry->vindirect.handle = ref_handle;
   entry->vindirect.ofs = rofs;
   entry->vindirect.len = rlen;
@@ -701,6 +683,7 @@ nv_table_realloc(NVTable *self, NVTable **new)
       memcpy(*new, self, sizeof(NVTable) + self->num_static_entries * sizeof(self->static_entries[0]) + self->num_dyn_entries * sizeof(NVDynValue));
       (*new)->ref_cnt = 1;
       (*new)->borrowed = FALSE;
+      (*new)->size = new_size;
 
       memmove(NV_TABLE_ADDR((*new), (*new)->size - (*new)->used),
               NV_TABLE_ADDR(self, old_size - self->used),
@@ -745,6 +728,9 @@ nv_table_clone(NVTable *self, gint additional_space)
   else
     new_size = self->size + (NV_TABLE_BOUND(additional_space));
 
+  if (new_size > NV_TABLE_MAX_BYTES)
+    new_size = NV_TABLE_MAX_BYTES;
+
   new = g_malloc(new_size);
   memcpy(new, self, sizeof(NVTable) + self->num_static_entries * sizeof(self->static_entries[0]) + self->num_dyn_entries * sizeof(NVDynValue));
   new->size = new_size;
