@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2002-2010 BalaBit IT Ltd, Budapest, Hungary
- * Copyright (c) 1998-2010 Balázs Scheidler
+ * Copyright (c) 2002-2013 BalaBit IT Ltd, Budapest, Hungary
+ * Copyright (c) 1998-2012 Balázs Scheidler
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -24,10 +24,13 @@
 
 #include "timeutils.h"
 #include "messages.h"
-#include "syslog-ng.h"
+#include "tls-support.h"
+#include "reloc.h"
+#include "pathutils.h"
 
 #include <ctype.h>
 #include <string.h>
+#include <iv.h>
 
 const char *month_names_abbrev[] =
 {
@@ -49,7 +52,6 @@ const char *weekday_names[] =
   "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"
 };
 
-static GTimeVal current_time_value;
 
 #define TZCACHE_SIZE 4
 #define TZCACHE_SIZE_MASK (TZCACHE_SIZE - 1)
@@ -69,14 +71,11 @@ static const gchar *time_zone_path_list[] =
 
 static const gchar *time_zone_basedir = NULL;
 
-static struct 
+typedef struct _TimeCache
 {
   time_t when;
-  long tzoff;
-} timezone_cache[TZCACHE_SIZE];
-static int timezone_cache_pos = 0;
-GStaticMutex cache_lock = G_STATIC_MUTEX_INIT;
-
+  struct tm tm;
+} TimeCache;
 
 static const gchar *
 get_time_zone_basedir(void)
@@ -85,13 +84,41 @@ get_time_zone_basedir(void)
 
   if (!time_zone_basedir)
     {
-      for (i = 0; time_zone_path_list[i] != NULL && !g_file_test(time_zone_path_list[i], G_FILE_TEST_IS_DIR); i++)
+      for (i = 0; time_zone_path_list[i] != NULL && !is_file_directory(get_installation_path_for(time_zone_path_list[i])); i++)
         ;
       time_zone_basedir = time_zone_path_list[i];
     }
   return time_zone_basedir;
 }
 
+TLS_BLOCK_START
+{
+  GTimeVal current_time_value;
+  struct iv_task invalidate_time_task;
+  TimeCache local_time_cache[64];
+  TimeCache gm_time_cache[64];
+  struct tm mktime_prev_tm;
+  time_t mktime_prev_time;
+}
+TLS_BLOCK_END;
+
+#define current_time_value   __tls_deref(current_time_value)
+#define invalidate_time_task __tls_deref(invalidate_time_task)
+#define local_time_cache     __tls_deref(local_time_cache)
+#define gm_time_cache        __tls_deref(gm_time_cache)
+#define mktime_prev_tm       __tls_deref(mktime_prev_tm)
+#define mktime_prev_time     __tls_deref(mktime_prev_time)
+
+#if !defined(HAVE_LOCALTIME_R) || !defined(HAVE_GMTIME_R)
+static GStaticMutex localtime_lock = G_STATIC_MUTEX_INIT;
+#endif
+
+void
+invalidate_cached_time(void)
+{
+  current_time_value.tv_sec = 0;
+}
+
 /*
  * this shuld replace the g_get_current_time and the g_source_get_current_time calls in the main thread
  * (log_msg_init, afinter_postpone_mark)
@@ -99,67 +126,112 @@ get_time_zone_basedir(void)
 void
 cached_g_current_time(GTimeVal *result)
 {
-  g_assert(result);
-  *result  = current_time_value;
+  if (current_time_value.tv_sec == 0)
+    {
+      g_get_current_time(&current_time_value);
+    }
+  *result = current_time_value;
+
+  if (iv_inited())
+    {
+      if (invalidate_time_task.handler == NULL)
+        {
+          IV_TASK_INIT(&invalidate_time_task);
+          invalidate_time_task.handler = (void (*)(void *)) invalidate_cached_time;
+        }
+      if (!iv_task_registered(&invalidate_time_task))
+        iv_task_register(&invalidate_time_task);
+    }
+  else
+    {
+      invalidate_cached_time();
+    }
 }
 
-/*
- * only our g_main_context_poll_func may call this
- */
-void
-update_g_current_time()
+time_t
+cached_g_current_time_sec()
 {
-  g_get_current_time(&current_time_value);
+  GTimeVal now;
+
+  cached_g_current_time(&now);
+  return now.tv_sec;
 }
 
 time_t
 cached_mktime(struct tm *tm)
 {
-  static struct tm prev_tm;
-  static time_t prev_time;
   time_t result;
 
-  g_static_mutex_lock(&cache_lock);
-
-  if (G_LIKELY(tm->tm_sec == prev_tm.tm_sec &&
-               tm->tm_min == prev_tm.tm_min &&
-               tm->tm_hour == prev_tm.tm_hour &&
-               tm->tm_mday == prev_tm.tm_mday &&
-               tm->tm_mon == prev_tm.tm_mon &&
-               tm->tm_year == prev_tm.tm_year))
+  if (G_LIKELY(tm->tm_sec == mktime_prev_tm.tm_sec &&
+               tm->tm_min == mktime_prev_tm.tm_min &&
+               tm->tm_hour == mktime_prev_tm.tm_hour &&
+               tm->tm_mday == mktime_prev_tm.tm_mday &&
+               tm->tm_mon == mktime_prev_tm.tm_mon &&
+               tm->tm_year == mktime_prev_tm.tm_year))
     {
-      result = prev_time;
-      g_static_mutex_unlock(&cache_lock);
+      result = mktime_prev_time;
       return result;
     }
-  g_static_mutex_unlock(&cache_lock);
   result = mktime(tm);
-  g_static_mutex_lock(&cache_lock);
-  prev_tm = *tm;
-  prev_time = result;
-  g_static_mutex_unlock(&cache_lock);
+  mktime_prev_tm = *tm;
+  mktime_prev_time = result;
   return result;
 }
 
 void
 cached_localtime(time_t *when, struct tm *tm)
 {
-  static time_t prev_when;
-  static struct tm prev_tm;
-  
-  g_static_mutex_lock(&cache_lock);
-  if (G_LIKELY(*when == prev_when))
+  guchar i = 0;
+
+  i = *when & 0x3F;
+  if (G_LIKELY(*when == local_time_cache[i].when))
     {
-      *tm = prev_tm;
-      g_static_mutex_unlock(&cache_lock);
+      *tm = local_time_cache[i].tm;
       return;
     }
-  g_static_mutex_unlock(&cache_lock);
-  localtime_r(when, tm);
-  g_static_mutex_lock(&cache_lock);
-  prev_tm = *tm;
-  prev_when = *when;
-  g_static_mutex_unlock(&cache_lock);
+  else
+    {
+#ifdef HAVE_LOCALTIME_R
+      localtime_r(when, tm);
+#else
+      struct tm *ltm;
+
+      g_static_mutex_lock(&localtime_lock);
+      ltm = localtime(when);
+      *tm = *ltm;
+      g_static_mutex_unlock(&localtime_lock);
+#endif
+      local_time_cache[i].tm = *tm;
+      local_time_cache[i].when = *when;
+    }
+}
+
+void
+cached_gmtime(time_t *when, struct tm *tm)
+{
+  guchar i = 0;
+
+  i = *when & 0x3F;
+  if (G_LIKELY(*when == gm_time_cache[i].when))
+    {
+      *tm = gm_time_cache[i].tm;
+      return;
+    }
+  else
+    {
+#ifdef HAVE_GMTIME_R
+      gmtime_r(when, tm);
+#else
+      struct tm *ltm;
+
+      g_static_mutex_lock(&localtime_lock);
+      ltm = gmtime(when);
+      *tm = *ltm;
+      g_static_mutex_unlock(&localtime_lock);
+#endif
+      gm_time_cache[i].tm = *tm;
+      gm_time_cache[i].when = *when;
+    }
 }
 
 /**
@@ -172,30 +244,20 @@ cached_localtime(time_t *when, struct tm *tm)
 long
 get_local_timezone_ofs(time_t when)
 {
+#ifdef HAVE_STRUCT_TM_TM_GMTOFF
+  struct tm ltm;
+
+  cached_localtime(&when, &ltm);
+  return ltm.tm_gmtoff;
+
+#else
+
+  struct tm gtm;
   struct tm ltm;
   long tzoff;
-  gint i;
   
-  g_static_mutex_lock(&cache_lock);
-  for (i = 0; i < TZCACHE_SIZE; i++)
-    {
-      if (timezone_cache[(timezone_cache_pos + i) & TZCACHE_SIZE_MASK].when == when)
-        {
-          tzoff = timezone_cache[(timezone_cache_pos + i) & TZCACHE_SIZE_MASK].tzoff;
-          g_static_mutex_unlock(&cache_lock);
-          return tzoff;
-        }
-    }
-  g_static_mutex_unlock(&cache_lock);
-  
-#if HAVE_STRUCT_TM_TM_GMTOFF && 0
-  localtime_r(&when, &ltm);
-  tzoff = ltm.tm_gmtoff;
-#else
-  struct tm gtm;
-  
-  localtime_r(&when, &ltm);
-  gmtime_r(&when, &gtm);
+  cached_localtime(&when, &ltm);
+  cached_gmtime(&when, &gtm);
 
   tzoff = (ltm.tm_hour - gtm.tm_hour) * 3600;
   tzoff += (ltm.tm_min - gtm.tm_min) * 60;
@@ -206,22 +268,16 @@ get_local_timezone_ofs(time_t when)
   else if (tzoff < 0 && (ltm.tm_year > gtm.tm_year || ltm.tm_mon > gtm.tm_mon || ltm.tm_mday > gtm.tm_mday))
     tzoff += 86400;
   
-#endif
-  g_static_mutex_lock(&cache_lock);
-  timezone_cache_pos = (timezone_cache_pos + 1) & TZCACHE_SIZE_MASK;
-  timezone_cache[timezone_cache_pos].when = when;
-  timezone_cache[timezone_cache_pos].tzoff = tzoff;
-  g_static_mutex_unlock(&cache_lock);
   return tzoff;
+#endif /* HAVE_STRUCT_TM_TM_GMTOFF */
 }
 
 
 void
 clean_time_cache()
 {
-  g_static_mutex_lock(&cache_lock);
-  memset(&timezone_cache, 0, sizeof(timezone_cache));
-  g_static_mutex_unlock(&cache_lock);
+  memset(&gm_time_cache, 0, sizeof(gm_time_cache));
+  memset(&local_time_cache, 0, sizeof(local_time_cache));
 }
 
 int
@@ -230,7 +286,40 @@ format_zone_info(gchar *buf, size_t buflen, glong gmtoff)
   return g_snprintf(buf, buflen, "%c%02ld:%02ld",
                           gmtoff < 0 ? '-' : '+',
                           (gmtoff < 0 ? -gmtoff : gmtoff) / 3600,
-                          (gmtoff % 3600) / 60);
+                          ((gmtoff < 0 ? -gmtoff : gmtoff) % 3600) / 60);
+}
+
+/**
+ * check_nanosleep:
+ *
+ * Check if nanosleep() is accurate enough for sub-millisecond sleeping. If
+ * it is not good enough, we're skipping the minor sleeps in LogReader to
+ * balance the cost of returning to the main loop (e.g.  we're always going
+ * to return to the main loop, instead of trying to wait for the writer).
+ **/
+gboolean
+check_nanosleep(void)
+{
+  struct timespec start, stop, sleep;
+  glong diff;
+  gint attempts;
+
+  for (attempts = 0; attempts < 3; attempts++)
+    {
+      clock_gettime(CLOCK_MONOTONIC, &start);
+      sleep.tv_sec = 0;
+      /* 0.1 msec */
+      sleep.tv_nsec = 1e5;
+
+      while (nanosleep(&sleep, &sleep) < 0)
+        ;
+
+      clock_gettime(CLOCK_MONOTONIC, &stop);
+      diff = timespec_diff_nsec(&stop, &start);
+      if (diff < 5e5)
+        return TRUE;
+    }
+  return FALSE;
 }
 
 /**
@@ -252,6 +341,31 @@ g_time_val_diff(GTimeVal *t1, GTimeVal *t2)
   return (t1->tv_sec - t2->tv_sec) * G_USEC_PER_SEC + (t1->tv_usec - t2->tv_usec);
 }
 
+void
+timespec_add_msec(struct timespec *ts, glong msec)
+{
+  ts->tv_sec += msec / 1000;
+  msec = msec % 1000;
+  ts->tv_nsec += msec * 1e6;
+  if (ts->tv_nsec > 1e9)
+    {
+      ts->tv_nsec -= 1e9;
+      ts->tv_sec++;
+    }
+}
+
+glong
+timspec_diff_msec(struct timespec *t1, struct timespec *t2)
+{
+  return (t1->tv_sec - t2->tv_sec) * 1e3 + (t1->tv_nsec - t2->tv_nsec) / 1e6;
+}
+
+glong
+timespec_diff_nsec(struct timespec *t1, struct timespec *t2)
+{
+  return (t1->tv_sec - t2->tv_sec) * 1e9 + (t1->tv_nsec - t2->tv_nsec);
+}
+
 /** Time zone file parser code **/
 
 /*
@@ -438,6 +552,9 @@ zone_info_parser(unsigned char **input, gboolean is64bitData, gint *version)
   gint64 *transition_times = NULL;
   guint8 *transition_types = NULL;
   gint32 *gmt_offsets = NULL;
+  gint64 isgmtcnt, isdstcnt, leapcnt, timecnt, typecnt, charcnt;
+  gboolean insertInitial = FALSE;
+
   buf = *input;
   *input += 4;
 
@@ -457,7 +574,7 @@ zone_info_parser(unsigned char **input, gboolean is64bitData, gint *version)
    * if '\0', we have just one copy of data,
    * if '2', there is additional 64 bit version at the end.
    */
-  if (buf[0] != 0 && buf[0] != '2')
+  if (buf[0] != 0 && buf[0] != '2' && buf[0] != '3')
     {
       msg_error("Error in the time zone file", 
                 evt_tag_str("message", "Bad Olson version info"), 
@@ -476,12 +593,12 @@ zone_info_parser(unsigned char **input, gboolean is64bitData, gint *version)
   *input += 15;
 
   /* Read array sizes */
-  gint64 isgmtcnt = readcoded32(input, 0, G_MAXINT64);
-  gint64 isdstcnt = readcoded32(input, 0, G_MAXINT64);
-  gint64 leapcnt  = readcoded32(input, 0, G_MAXINT64);
-  gint64 timecnt  = readcoded32(input, 0, G_MAXINT64);
-  gint64 typecnt  = readcoded32(input, 0, G_MAXINT64);
-  gint64 charcnt  = readcoded32(input, 0, G_MAXINT64);
+  isgmtcnt = readcoded32(input, 0, G_MAXINT64);
+  isdstcnt = readcoded32(input, 0, G_MAXINT64);
+  leapcnt  = readcoded32(input, 0, G_MAXINT64);
+  timecnt  = readcoded32(input, 0, G_MAXINT64);
+  typecnt  = readcoded32(input, 0, G_MAXINT64);
+  charcnt  = readcoded32(input, 0, G_MAXINT64);
 
   /* 
    * Confirm sizes that we assume to be equal.  These assumptions
@@ -538,13 +655,18 @@ zone_info_parser(unsigned char **input, gboolean is64bitData, gint *version)
  /* Read types (except for the isstd and isgmt flags, which come later (why??)) */
   for (i = 0; i<typecnt; ++i)
     {
+      gint offs = 24;
+
+      if (*version == 3)
+        offs = 167;
+
       gmt_offsets[i] = readcoded32(input, G_MININT64, G_MAXINT64);
-      if (gmt_offsets[i] > 24 * 60 * 60 || gmt_offsets[i] < -1 * 24 * 60 * 60)
+      if (gmt_offsets[i] > offs * 60 * 60 || gmt_offsets[i] < -1 * offs * 60 * 60)
         {
           msg_warning("Error in the time zone file", 
                       evt_tag_str("message", "Illegal gmtoffset number"), 
                       evt_tag_int("val", gmt_offsets[i]), 
-                      evt_tag_printf("expected", "[%d, %d]", -1 * 24 * 60 * 60, 24 * 60 * 60), 
+                      evt_tag_printf("expected", "[%d, %d]", -1 * offs * 60 * 60, offs * 60 * 60), 
                       NULL);
           goto error;
         }
@@ -568,7 +690,7 @@ zone_info_parser(unsigned char **input, gboolean is64bitData, gint *version)
     }
  
   /* Build transitions vector out of corresponding times and types. */
-  gboolean insertInitial = FALSE;
+  insertInitial = FALSE;
   if (is64bitData)
     {
       if (timecnt > 0)
@@ -726,7 +848,7 @@ zone_info_read(const gchar *zonename, ZoneInfo **zone, ZoneInfo **zone64)
   if (byte_read == -1)
     {
       msg_error("Failed to read the time zone file", evt_tag_str("filename", filename), NULL);
-      g_mapped_file_free(file_map);
+      g_mapped_file_unref(file_map);
       g_free(filename);
       return FALSE;
     }
@@ -739,7 +861,7 @@ zone_info_read(const gchar *zonename, ZoneInfo **zone, ZoneInfo **zone64)
       *zone64 = zone_info_parser(&buff, TRUE, &version);
     }
 
-  g_mapped_file_free(file_map);
+  g_mapped_file_unref(file_map);
     g_free(filename);
   return TRUE;
 }
@@ -813,4 +935,3 @@ time_zone_info_free(TimeZoneInfo *self)
   zone_info_free(self->zone64);
   g_free(self);
 }
-
