@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 2012 Nagy, Attila <bra@fsn.hu>
- * Copyright (c) 2012-2013 BalaBit IT Ltd, Budapest, Hungary
- * Copyright (c) 2012-2013 Gergely Nagy <algernon@balabit.hu>
+ * Copyright (c) 2012-2014 BalaBit IT Ltd, Budapest, Hungary
+ * Copyright (c) 2012-2014 Gergely Nagy <algernon@balabit.hu>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -27,17 +27,19 @@
 #include "plugin.h"
 #include "messages.h"
 #include "misc.h"
-#include "stats.h"
+#include "stats/stats-registry.h"
 #include "nvtable.h"
 #include "logqueue.h"
 #include "scratch-buffers.h"
+#include "plugin-types.h"
+#include "logthrdestdrv.h"
 
 #include <amqp.h>
 #include <amqp_framing.h>
 
 typedef struct
 {
-  LogDestDriver super;
+  LogThrDestDriver super;
 
   /* Shared between main/writer; only read by the writer, never written */
   gchar *exchange;
@@ -55,30 +57,13 @@ typedef struct
   gchar *user;
   gchar *password;
 
-  time_t time_reopen;
-
-  StatsCounterItem *dropped_messages;
-  StatsCounterItem *stored_messages;
-
+  LogTemplateOptions template_options;
   ValuePairs *vp;
 
-  /* Thread related stuff; shared */
-  GThread *writer_thread;
-  GMutex *queue_mutex;
-  GMutex *suspend_mutex;
-  GCond *writer_thread_wakeup_cond;
-
-  gboolean writer_thread_terminate;
-  gboolean writer_thread_suspended;
-  GTimeVal writer_thread_suspend_target;
-
-  LogQueue *queue;
-
   /* Writer-only stuff */
   amqp_connection_state_t conn;
   amqp_table_entry_t *entries;
   gint32 max_entries;
-  gint32 seq_num;
 } AMQPDestDriver;
 
 /*
@@ -190,17 +175,26 @@ afamqp_dd_set_value_pairs(LogDriver *d, ValuePairs *vp)
   AMQPDestDriver *self = (AMQPDestDriver *) d;
 
   if (self->vp)
-    value_pairs_free(self->vp);
+    value_pairs_unref(self->vp);
   self->vp = vp;
 }
 
+LogTemplateOptions *
+afamqp_dd_get_template_options(LogDriver *s)
+{
+  AMQPDestDriver *self = (AMQPDestDriver *) s;
+
+  return &self->template_options;
+}
+
 /*
  * Utilities
  */
 
 static gchar *
-afamqp_dd_format_stats_instance(AMQPDestDriver *self)
+afamqp_dd_format_stats_instance(LogThrDestDriver *s)
 {
+  AMQPDestDriver *self = (AMQPDestDriver *) s;
   static gchar persist_name[1024];
 
   g_snprintf(persist_name, sizeof(persist_name), "amqp,%s,%s,%u,%s,%s",
@@ -210,8 +204,9 @@ afamqp_dd_format_stats_instance(AMQPDestDriver *self)
 }
 
 static gchar *
-afamqp_dd_format_persist_name(AMQPDestDriver *self)
+afamqp_dd_format_persist_name(LogThrDestDriver *s)
 {
+  AMQPDestDriver *self = (AMQPDestDriver *) s;
   static gchar persist_name[1024];
 
   g_snprintf(persist_name, sizeof(persist_name), "afamqp(%s,%s,%u,%s,%s)",
@@ -221,19 +216,9 @@ afamqp_dd_format_persist_name(AMQPDestDriver *self)
 }
 
 static void
-afamqp_dd_suspend(AMQPDestDriver *self)
+afamqp_dd_disconnect(LogThrDestDriver *s)
 {
-  self->writer_thread_suspended = TRUE;
-  g_get_current_time(&self->writer_thread_suspend_target);
-  g_time_val_add(&self->writer_thread_suspend_target,
-                 self->time_reopen * 1000000);
-}
-
-static void
-afamqp_dd_disconnect(AMQPDestDriver *self)
-{
-  if (!self->conn)
-    return;
+  AMQPDestDriver *self = (AMQPDestDriver *)s;
 
   amqp_channel_close(self->conn, 1, AMQP_REPLY_SUCCESS);
   amqp_connection_close(self->conn, AMQP_REPLY_SUCCESS);
@@ -251,23 +236,23 @@ afamqp_is_ok(AMQPDestDriver *self, gchar *context, amqp_rpc_reply_t ret)
 
     case AMQP_RESPONSE_NONE:
       msg_error(context,
-                evt_tag_str("driver", self->super.super.id),
+                evt_tag_str("driver", self->super.super.super.id),
                 evt_tag_str("error", "missing RPC reply type"),
-                evt_tag_int("time_reopen", self->time_reopen),
+                evt_tag_int("time_reopen", self->super.time_reopen),
                 NULL);
-      afamqp_dd_suspend(self);
+      log_threaded_dest_driver_suspend(&self->super);
       return FALSE;
 
     case AMQP_RESPONSE_LIBRARY_EXCEPTION:
       {
         gchar *errstr = amqp_error_string(ret.library_error);
         msg_error(context,
-                  evt_tag_str("driver", self->super.super.id),
+                  evt_tag_str("driver", self->super.super.super.id),
                   evt_tag_str("error", errstr),
-                  evt_tag_int("time_reopen", self->time_reopen),
+                  evt_tag_int("time_reopen", self->super.time_reopen),
                   NULL);
         g_free (errstr);
-        afamqp_dd_suspend(self);
+        log_threaded_dest_driver_suspend(&self->super);
         return FALSE;
       }
 
@@ -279,13 +264,13 @@ afamqp_is_ok(AMQPDestDriver *self, gchar *context, amqp_rpc_reply_t ret)
             amqp_connection_close_t *m =
               (amqp_connection_close_t *) ret.reply.decoded;
             msg_error(context,
-                      evt_tag_str("driver", self->super.super.id),
+                      evt_tag_str("driver", self->super.super.super.id),
                       evt_tag_str("error", "server connection error"),
                       evt_tag_int("code", m->reply_code),
                       evt_tag_str("text", m->reply_text.bytes),
-                      evt_tag_int("time_reopen", self->time_reopen),
+                      evt_tag_int("time_reopen", self->super.time_reopen),
                       NULL);
-            afamqp_dd_suspend(self);
+            log_threaded_dest_driver_suspend(&self->super);
             return FALSE;
           }
         case AMQP_CHANNEL_CLOSE_METHOD:
@@ -293,23 +278,23 @@ afamqp_is_ok(AMQPDestDriver *self, gchar *context, amqp_rpc_reply_t ret)
             amqp_channel_close_t *m =
               (amqp_channel_close_t *) ret.reply.decoded;
             msg_error(context,
-                      evt_tag_str("driver", self->super.super.id),
+                      evt_tag_str("driver", self->super.super.super.id),
                       evt_tag_str("error", "server channel error"),
                       evt_tag_int("code", m->reply_code),
                       evt_tag_str("text", m->reply_text.bytes),
-                      evt_tag_int("time_reopen", self->time_reopen),
+                      evt_tag_int("time_reopen", self->super.time_reopen),
                       NULL);
-            afamqp_dd_suspend(self);
+            log_threaded_dest_driver_suspend(&self->super);
             return FALSE;
           }
         default:
           msg_error(context,
-                    evt_tag_str("driver", self->super.super.id),
+                    evt_tag_str("driver", self->super.super.super.id),
                     evt_tag_str("error", "unknown server error"),
                     evt_tag_printf("method id", "0x%08X", ret.reply.id),
-                    evt_tag_int("time_reopen", self->time_reopen),
+                    evt_tag_int("time_reopen", self->super.time_reopen),
                     NULL);
-          afamqp_dd_suspend(self);
+          log_threaded_dest_driver_suspend(&self->super);
           return FALSE;
         }
       return FALSE;
@@ -336,9 +321,9 @@ afamqp_dd_connect(AMQPDestDriver *self, gboolean reconnect)
     {
       gchar *errstr = amqp_error_string(-sockfd);
       msg_error("Error connecting to AMQP server",
-                evt_tag_str("driver", self->super.super.id),
+                evt_tag_str("driver", self->super.super.super.id),
                 evt_tag_str("error", errstr),
-                evt_tag_int("time_reopen", self->time_reopen),
+                evt_tag_int("time_reopen", self->super.time_reopen),
                 NULL);
       g_free(errstr);
       return FALSE;
@@ -366,7 +351,7 @@ afamqp_dd_connect(AMQPDestDriver *self, gboolean reconnect)
     }
 
   msg_debug ("Connecting to AMQP succeeded",
-             evt_tag_str("driver", self->super.super.id),
+             evt_tag_str("driver", self->super.super.super.id),
              NULL);
 
   return TRUE;
@@ -377,7 +362,8 @@ afamqp_dd_connect(AMQPDestDriver *self, gboolean reconnect)
  */
 
 static gboolean
-afamqp_vp_foreach(const gchar *name, const gchar *value,
+afamqp_vp_foreach(const gchar *name,
+                  TypeHint type, const gchar *value,
                   gpointer user_data)
 {
   amqp_table_entry_t **entries = (amqp_table_entry_t **) ((gpointer *)user_data)[0];
@@ -406,14 +392,15 @@ afamqp_worker_publish(AMQPDestDriver *self, LogMessage *msg)
   amqp_table_t table;
   amqp_basic_properties_t props;
   gboolean success = TRUE;
-  ScratchBuffer *routing_key = scratch_buffer_acquire();
-  ScratchBuffer *body = scratch_buffer_acquire();
+  SBGString *routing_key = sb_gstring_acquire();
+  SBGString *body = sb_gstring_acquire();
   amqp_bytes_t body_bytes = amqp_cstring_bytes("");
 
   gpointer user_data[] = { &self->entries, &pos, &self->max_entries };
 
-  value_pairs_foreach(self->vp, afamqp_vp_foreach, msg, self->seq_num,
-                      user_data);
+  value_pairs_foreach(self->vp, afamqp_vp_foreach, msg,
+                      self->super.seq_num,
+                      LTZ_SEND, &self->template_options, user_data);
 
   table.num_entries = pos;
   table.entries = self->entries;
@@ -425,26 +412,29 @@ afamqp_worker_publish(AMQPDestDriver *self, LogMessage *msg)
   props.headers = table;
 
   log_template_format(self->routing_key_template, msg, NULL, LTZ_LOCAL,
-                      self->seq_num, NULL, sb_string(routing_key));
+                      self->super.seq_num,
+                      NULL, sb_gstring_string(routing_key));
 
   if (self->body_template)
     {
       log_template_format(self->body_template, msg, NULL, LTZ_LOCAL,
-                          self->seq_num, NULL, sb_string(body));
-      body_bytes = amqp_cstring_bytes(sb_string(body)->str);
+                          self->super.seq_num,
+                          NULL, sb_gstring_string(body));
+      body_bytes = amqp_cstring_bytes(sb_gstring_string(body)->str);
     }
 
   ret = amqp_basic_publish(self->conn, 1, amqp_cstring_bytes(self->exchange),
-                           amqp_cstring_bytes(sb_string(routing_key)->str),
+                           amqp_cstring_bytes(sb_gstring_string(routing_key)->str),
                            0, 0, &props, body_bytes);
 
-  scratch_buffer_release(routing_key);
-  scratch_buffer_release(body);
+  sb_gstring_release(routing_key);
+  sb_gstring_release(body);
 
   if (ret < 0)
     {
       msg_error("Network error while inserting into AMQP server",
-                evt_tag_int("time_reopen", self->time_reopen), NULL);
+                evt_tag_str("driver", self->super.super.super.id),
+                evt_tag_int("time_reopen", self->super.time_reopen), NULL);
       success = FALSE;
     }
 
@@ -457,117 +447,32 @@ afamqp_worker_publish(AMQPDestDriver *self, LogMessage *msg)
   return success;
 }
 
-static gboolean
-afamqp_worker_insert(AMQPDestDriver *self)
+static worker_insert_result_t
+afamqp_worker_insert(LogThrDestDriver *s, LogMessage *msg)
 {
-  gboolean success;
-  LogMessage *msg;
-  LogPathOptions path_options = LOG_PATH_OPTIONS_INIT;
+  AMQPDestDriver *self = (AMQPDestDriver *)s;
 
-  afamqp_dd_connect(self, TRUE);
+  if (!afamqp_dd_connect(self, TRUE))
+    return WORKER_INSERT_RESULT_NOT_CONNECTED;
 
-  success = log_queue_pop_head(self->queue, &msg, &path_options, FALSE, FALSE);
-  if (!success)
-    return TRUE;
+  if (!afamqp_worker_publish (self, msg))
+    return WORKER_INSERT_RESULT_ERROR;
 
-  msg_set_context(msg);
-  success = afamqp_worker_publish (self, msg);
-  msg_set_context(NULL);
-
-  if (success)
-    {
-      stats_counter_inc(self->stored_messages);
-      step_sequence_number(&self->seq_num);
-      log_msg_ack(msg, &path_options);
-      log_msg_unref(msg);
-    }
-  else
-    {
-      g_mutex_lock(self->queue_mutex);
-      log_queue_push_head(self->queue, msg, &path_options);
-      g_mutex_unlock(self->queue_mutex);
-    }
-
-  return success;
+  return WORKER_INSERT_RESULT_SUCCESS;
 }
 
 static void
-afamqp_dd_message_became_available_in_the_queue(gpointer user_data)
+afamqp_worker_thread_init(LogThrDestDriver *d)
 {
-  AMQPDestDriver *self = (AMQPDestDriver *) user_data;
-
-  g_mutex_lock(self->suspend_mutex);
-  g_cond_signal(self->writer_thread_wakeup_cond);
-  g_mutex_unlock(self->suspend_mutex);
-}
-
-static gpointer
-afamqp_worker_thread(gpointer arg)
-{
-  AMQPDestDriver *self = (AMQPDestDriver *) arg;
-
-  msg_debug("Worker thread started",
-            evt_tag_str("driver", self->super.super.id), NULL);
+  AMQPDestDriver *self = (AMQPDestDriver *)d;
 
   afamqp_dd_connect(self, FALSE);
-
-  while (!self->writer_thread_terminate)
-    {
-      g_mutex_lock(self->suspend_mutex);
-      if (self->writer_thread_suspended)
-        {
-          g_cond_timed_wait(self->writer_thread_wakeup_cond,
-                            self->suspend_mutex, &self->writer_thread_suspend_target);
-          self->writer_thread_suspended = FALSE;
-          g_mutex_unlock(self->suspend_mutex);
-        }
-      else if (!log_queue_check_items(self->queue, NULL, afamqp_dd_message_became_available_in_the_queue, self, NULL))
-	{
-	  g_cond_wait(self->writer_thread_wakeup_cond, self->suspend_mutex);
-	  g_mutex_unlock(self->suspend_mutex);
-	}
-      else
-        g_mutex_unlock(self->suspend_mutex);
-
-      if (self->writer_thread_terminate)
-        break;
-
-      if (!afamqp_worker_insert(self))
-        {
-          afamqp_dd_disconnect(self);
-          afamqp_dd_suspend(self);
-        }
-    }
-
-  afamqp_dd_disconnect(self);
-
-  msg_debug("Worker thread finished",
-            evt_tag_str("driver", self->super.super.id), NULL);
-
-  return NULL;
 }
 
 /*
  * Main thread
  */
 
-static void
-afamqp_dd_start_thread(AMQPDestDriver *self)
-{
-  self->writer_thread = create_worker_thread(afamqp_worker_thread, self,
-                                             TRUE, NULL);
-}
-
-static void
-afamqp_dd_stop_thread(AMQPDestDriver *self)
-{
-  self->writer_thread_terminate = TRUE;
-  g_mutex_lock(self->queue_mutex);
-  g_cond_signal(self->writer_thread_wakeup_cond);
-  g_mutex_unlock(self->queue_mutex);
-  g_thread_join(self->writer_thread);
-}
-
 static gboolean
 afamqp_dd_init(LogPipe *s)
 {
@@ -577,25 +482,16 @@ afamqp_dd_init(LogPipe *s)
   if (!log_dest_driver_init_method(s))
     return FALSE;
 
-  if (cfg)
-    self->time_reopen = cfg->time_reopen;
-
-  if (!self->vp)
-    {
-      self->vp = value_pairs_new();
-      value_pairs_add_scope(self->vp, "selected-macros");
-      value_pairs_add_scope(self->vp, "nv-pairs");
-      value_pairs_add_scope(self->vp, "sdata");
-    }
-
   if (!self->user || !self->password)
     {
       msg_error("Error initializing AMQP destination: username and password MUST be set!",
-                evt_tag_str("driver", self->super.super.id),
+                evt_tag_str("driver", self->super.super.super.id),
                 NULL);
       return FALSE;
     }
 
+  log_template_options_init(&self->template_options, cfg);
+
   msg_verbose("Initializing AMQP destination",
               evt_tag_str("vhost", self->vhost),
               evt_tag_str("host", self->host),
@@ -604,45 +500,7 @@ afamqp_dd_init(LogPipe *s)
               evt_tag_str("exchange_type", self->exchange_type),
               NULL);
 
-  self->queue = log_dest_driver_acquire_queue(&self->super, afamqp_dd_format_persist_name(self));
-
-  stats_lock();
-  stats_register_counter(0, SCS_AMQP | SCS_DESTINATION,
-                         self->super.super.id, afamqp_dd_format_stats_instance(self),
-                         SC_TYPE_STORED, &self->stored_messages);
-  stats_register_counter(0, SCS_AMQP | SCS_DESTINATION,
-                         self->super.super.id, afamqp_dd_format_stats_instance(self),
-                         SC_TYPE_DROPPED, &self->dropped_messages);
-  stats_unlock();
-
-  log_queue_set_counters(self->queue, self->stored_messages,
-                         self->dropped_messages);
-  afamqp_dd_start_thread(self);
-
-  return TRUE;
-}
-
-static gboolean
-afamqp_dd_deinit(LogPipe *s)
-{
-  AMQPDestDriver *self = (AMQPDestDriver *) s;
-
-  afamqp_dd_stop_thread(self);
-  log_queue_reset_parallel_push(self->queue);
-
-  log_queue_set_counters(self->queue, NULL, NULL);
-  stats_lock();
-  stats_unregister_counter(SCS_AMQP | SCS_DESTINATION,
-                           self->super.super.id, afamqp_dd_format_stats_instance(self),
-                           SC_TYPE_STORED, &self->stored_messages);
-  stats_unregister_counter(SCS_AMQP | SCS_DESTINATION,
-                           self->super.super.id, afamqp_dd_format_stats_instance(self),
-                           SC_TYPE_DROPPED, &self->dropped_messages);
-  stats_unlock();
-  if (!log_dest_driver_deinit_method(s))
-    return FALSE;
-
-  return TRUE;
+  return log_threaded_dest_driver_start(s);
 }
 
 static void
@@ -650,12 +508,7 @@ afamqp_dd_free(LogPipe *d)
 {
   AMQPDestDriver *self = (AMQPDestDriver *) d;
 
-  g_mutex_free(self->suspend_mutex);
-  g_mutex_free(self->queue_mutex);
-  g_cond_free(self->writer_thread_wakeup_cond);
-
-  if (self->queue)
-    log_queue_unref(self->queue);
+  log_template_options_destroy(&self->template_options);
 
   g_free(self->exchange);
   g_free(self->exchange_type);
@@ -667,23 +520,9 @@ afamqp_dd_free(LogPipe *d)
   g_free(self->vhost);
   g_free(self->entries);
   if (self->vp)
-    value_pairs_free(self->vp);
-  log_dest_driver_free(d);
-}
+    value_pairs_unref(self->vp);
 
-static void
-afamqp_dd_queue(LogPipe *s, LogMessage *msg,
-                const LogPathOptions *path_options, gpointer user_data)
-{
-  AMQPDestDriver *self = (AMQPDestDriver *) s;
-  LogPathOptions local_options;
-
-  if (!path_options->flow_control_requested)
-    path_options = log_msg_break_ack(msg, path_options, &local_options);
-
-  log_msg_add_ack(msg, path_options);
-  log_queue_push_tail(self->queue, log_msg_ref(msg), path_options);
-  log_dest_driver_queue_method(s, msg, path_options, user_data);
+  log_threaded_dest_driver_free(d);
 }
 
 /*
@@ -691,17 +530,24 @@ afamqp_dd_queue(LogPipe *s, LogMessage *msg,
  */
 
 LogDriver *
-afamqp_dd_new(void)
+afamqp_dd_new(GlobalConfig *cfg)
 {
   AMQPDestDriver *self = g_new0(AMQPDestDriver, 1);
 
-  log_dest_driver_init_instance(&self->super);
-  self->super.super.super.init = afamqp_dd_init;
-  self->super.super.super.deinit = afamqp_dd_deinit;
-  self->super.super.super.queue = afamqp_dd_queue;
-  self->super.super.super.free_fn = afamqp_dd_free;
+  log_threaded_dest_driver_init_instance(&self->super, cfg);
 
-  self->routing_key_template = log_template_new(configuration, NULL);
+  self->super.super.super.super.init = afamqp_dd_init;
+  self->super.super.super.super.free_fn = afamqp_dd_free;
+
+  self->super.worker.thread_init = afamqp_worker_thread_init;
+  self->super.worker.disconnect = afamqp_dd_disconnect;
+  self->super.worker.insert = afamqp_worker_insert;
+
+  self->super.format.stats_instance = afamqp_dd_format_stats_instance;
+  self->super.format.persist_name = afamqp_dd_format_persist_name;
+  self->super.stats_source = SCS_AMQP;
+
+  self->routing_key_template = log_template_new(cfg, NULL);
 
   afamqp_dd_set_vhost((LogDriver *) self, "/");
   afamqp_dd_set_host((LogDriver *) self, "127.0.0.1");
@@ -712,15 +558,12 @@ afamqp_dd_new(void)
   afamqp_dd_set_persistent((LogDriver *) self, TRUE);
   afamqp_dd_set_exchange_declare((LogDriver *) self, FALSE);
 
-  init_sequence_number(&self->seq_num);
-
-  self->writer_thread_wakeup_cond = g_cond_new();
-  self->suspend_mutex = g_mutex_new();
-  self->queue_mutex = g_mutex_new();
-
   self->max_entries = 256;
   self->entries = g_new(amqp_table_entry_t, self->max_entries);
 
+  log_template_options_defaults(&self->template_options);
+  afamqp_dd_set_value_pairs(&self->super.super.super, value_pairs_new_default(cfg));
+
   return (LogDriver *) self;
 }
 
