@@ -27,11 +27,16 @@
 #include "misc.h"
 #include "serialize.h"
 #include "gprocess.h"
-#include "stats.h"
-#include "mainloop.h"
-#include "logproto-text-client.h"
+#include "stats/stats-registry.h"
+#include "mainloop-call.h"
+#include "transport/transport-file.h"
+#include "logproto/logproto-text-client.h"
 #include "logproto-file-writer.h"
+#include "transport/transport-file.h"
+#include "transport/transport-pipe.h"
+#include "compat/lfs.h"
 
+#include <iv.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -41,6 +46,9 @@
 #include <time.h>
 #include <stdlib.h>
 
+#define DEFAULT_DW_REOPEN_FLAGS (O_WRONLY | O_CREAT | O_NOCTTY | O_NONBLOCK | O_LARGEFILE | O_APPEND)
+#define DEFAULT_DW_REOPEN_FLAGS_PIPE (O_RDWR | O_NOCTTY | O_NONBLOCK | O_LARGEFILE)
+
 /*
  * Threading notes:
  *
@@ -84,7 +92,7 @@ struct _AFFileDestWriter
   GStaticMutex lock;
   AFFileDestDriver *owner;
   gchar *filename;
-  LogPipe *writer;
+  LogWriter *writer;
   time_t last_msg_stamp;
   time_t last_open_stamp;
   time_t time_reopen;
@@ -123,7 +131,7 @@ affile_dw_reap(gpointer s)
   main_loop_assert_main_thread();
 
   g_static_mutex_lock(&self->lock);
-  if (!log_writer_has_pending_writes((LogWriter *) self->writer) &&
+  if (!log_writer_has_pending_writes(self->writer) &&
       !self->queue_pending &&
       (cached_g_current_time_sec() - self->last_msg_stamp) >= self->owner->time_reap)
     {
@@ -141,10 +149,16 @@ affile_dw_reap(gpointer s)
     }
 }
 
+static gboolean
+_affile_dw_reopen_file(AFFileDestWriter *self, gchar *name, gint *fd)
+{
+  return affile_open_file(name, &self->owner->file_open_options, &self->owner->file_perm_options, fd);
+}
+
 static gboolean
 affile_dw_reopen(AFFileDestWriter *self)
 {
-  int fd, flags;
+  int fd;
   struct stat st;
 
   self->last_open_stamp = self->last_msg_stamp;
@@ -159,21 +173,14 @@ affile_dw_reopen(AFFileDestWriter *self)
       unlink(self->filename);
     }
 
-  if (self->owner->flags & AFFILE_PIPE)
-    flags = O_RDWR | O_NOCTTY | O_NONBLOCK | O_LARGEFILE;
-  else
-    flags = O_WRONLY | O_CREAT | O_NOCTTY | O_NONBLOCK | O_LARGEFILE | O_APPEND;
-
-
-  if (affile_open_file(self->filename, flags, &self->owner->file_perm_options,
-                       !!(self->owner->flags & AFFILE_CREATE_DIRS), FALSE, !!(self->owner->flags & AFFILE_PIPE), &fd))
+  if (_affile_dw_reopen_file(self, self->filename, &fd))
     {
       log_writer_reopen(self->writer,
-                        self->owner->flags & AFFILE_PIPE
+                        self->owner->file_open_options.is_pipe
                         ? log_proto_text_client_new(log_transport_pipe_new(fd), &self->owner->writer_options.proto_options.super)
                         : log_proto_file_writer_new(log_transport_file_new(fd), &self->owner->writer_options.proto_options.super,
                                                     self->owner->writer_options.flush_lines,
-                                                    (self->owner->flags & AFFILE_FSYNC)));
+                                                    self->owner->use_fsync));
 
       main_loop_call((void * (*)(void *)) affile_dw_arm_reaper, self, TRUE);
     }
@@ -207,23 +214,27 @@ affile_dw_init(LogPipe *s)
       guint32 flags;
 
       flags = LW_FORMAT_FILE |
-        ((self->owner->flags & AFFILE_PIPE) ? 0 : LW_SOFT_FLOW_CONTROL);
+        (self->owner->file_open_options.is_pipe ? 0 : LW_SOFT_FLOW_CONTROL);
 
-      self->writer = log_writer_new(flags);
+      self->writer = log_writer_new(flags, cfg);
     }
-  log_writer_set_options((LogWriter *) self->writer, s, &self->owner->writer_options, 1,
-                         self->owner->flags & AFFILE_PIPE ? SCS_PIPE : SCS_FILE,
-                         self->owner->super.super.id, self->filename);
+  log_writer_set_options(self->writer,
+                         s,
+                         &self->owner->writer_options,
+                         STATS_LEVEL1,
+                         self->owner->file_open_options.is_pipe ? SCS_PIPE : SCS_FILE,
+                         self->owner->super.super.id,
+                         self->filename);
   log_writer_set_queue(self->writer, log_dest_driver_acquire_queue(&self->owner->super, affile_dw_format_persist_name(self)));
 
-  if (!log_pipe_init(self->writer, NULL))
+  if (!log_pipe_init((LogPipe *) self->writer))
     {
       msg_error("Error initializing log writer", NULL);
-      log_pipe_unref(self->writer);
+      log_pipe_unref((LogPipe *) self->writer);
       self->writer = NULL;
       return FALSE;
     }
-  log_pipe_append(&self->super, self->writer);
+  log_pipe_append(&self->super, (LogPipe *) self->writer);
 
   return affile_dw_reopen(self);
 }
@@ -236,7 +247,7 @@ affile_dw_deinit(LogPipe *s)
   main_loop_assert_main_thread();
   if (self->writer)
     {
-      log_pipe_deinit(self->writer);
+      log_pipe_deinit((LogPipe *) self->writer);
     }
   log_dest_driver_release_queue(&self->owner->super, log_writer_get_queue(self->writer));
   log_writer_set_queue(self->writer, NULL);
@@ -261,7 +272,7 @@ affile_dw_queue(LogPipe *s, LogMessage *lm, const LogPathOptions *path_options,
   if (self->last_open_stamp == 0)
     self->last_open_stamp = self->last_msg_stamp;
 
-  if (!log_writer_opened((LogWriter *) self->writer) &&
+  if (!log_writer_opened(self->writer) &&
       !self->reopen_pending &&
       (self->last_open_stamp < self->last_msg_stamp - self->time_reopen))
     {
@@ -280,13 +291,25 @@ affile_dw_queue(LogPipe *s, LogMessage *lm, const LogPathOptions *path_options,
 static void
 affile_dw_set_owner(AFFileDestWriter *self, AFFileDestDriver *owner)
 {
+  GlobalConfig *cfg = log_pipe_get_config(&owner->super.super.super);
+
   if (self->owner)
     log_pipe_unref(&self->owner->super.super.super);
   log_pipe_ref(&owner->super.super.super);
   self->owner = owner;
+
+  log_pipe_set_config(&self->super, cfg);
   if (self->writer)
-    log_writer_set_options((LogWriter *) self->writer, &self->super, &owner->writer_options, 1, SCS_FILE, self->owner->super.super.id, self->filename);
-  
+    {
+      log_pipe_set_config((LogPipe *) self->writer, cfg);
+      log_writer_set_options(self->writer,
+                             &self->super,
+                             &owner->writer_options,
+                             STATS_LEVEL1,
+                             SCS_FILE,
+                             self->owner->super.super.id,
+                             self->filename);
+    }
 }
 
 static void
@@ -294,8 +317,7 @@ affile_dw_free(LogPipe *s)
 {
   AFFileDestWriter *self = (AFFileDestWriter *) s;
   
-  g_static_mutex_free(&self->lock);
-  log_pipe_unref(self->writer);
+  log_pipe_unref((LogPipe *) self->writer);
   self->writer = NULL;
   g_free(self->filename);
   log_pipe_unref(&self->owner->super.super.super);
@@ -307,7 +329,7 @@ affile_dw_new(AFFileDestDriver *owner, const gchar *filename)
 {
   AFFileDestWriter *self = g_new0(AFFileDestWriter, 1);
   
-  log_pipe_init_instance(&self->super);
+  log_pipe_init_instance(&self->super, owner->super.super.super.cfg);
 
   self->super.init = affile_dw_init;
   self->super.deinit = affile_dw_deinit;
@@ -333,10 +355,7 @@ affile_dd_set_create_dirs(LogDriver *s, gboolean create_dirs)
 {
   AFFileDestDriver *self = (AFFileDestDriver *) s;
   
-  if (create_dirs)
-    self->flags |= AFFILE_CREATE_DIRS;
-  else 
-    self->flags &= ~AFFILE_CREATE_DIRS;
+  self->file_open_options.create_dirs = create_dirs;
 }
 
 void 
@@ -351,18 +370,8 @@ void
 affile_dd_set_fsync(LogDriver *s, gboolean fsync)
 {
   AFFileDestDriver *self = (AFFileDestDriver *) s;
-  if (fsync)
-    self->flags |= AFFILE_FSYNC;
-  else
-    self->flags &= ~AFFILE_FSYNC;
-}
 
-void
-affile_dd_set_local_time_zone(LogDriver *s, const gchar *local_time_zone)
-{
-  AFFileDestDriver *self = (AFFileDestDriver *) s;
-
-  self->local_time_zone = g_strdup(local_time_zone);
+  self->use_fsync = fsync;
 }
 
 static inline gchar *
@@ -379,7 +388,7 @@ affile_dd_reap_writer(AFFileDestDriver *self, AFFileDestWriter *dw)
 {
   main_loop_assert_main_thread();
   
-  if ((self->flags & AFFILE_NO_EXPAND) == 0)
+  if (self->filename_is_a_template)
     {
       g_static_mutex_lock(&self->lock);
       /* remove from hash table */
@@ -415,7 +424,12 @@ affile_dd_reuse_writer(gpointer key, gpointer value, gpointer user_data)
   AFFileDestWriter *writer = (AFFileDestWriter *) value;
   
   affile_dw_set_owner(writer, self);
-  log_pipe_init(&writer->super, NULL);
+  if (!log_pipe_init(&writer->super))
+    {
+      affile_dw_set_owner(writer, NULL);
+      log_pipe_unref(&writer->super);
+      g_hash_table_remove(self->writer_hash, key);
+    }
 }
 
 
@@ -429,15 +443,14 @@ affile_dd_init(LogPipe *s)
     return FALSE;
 
   if (cfg->create_dirs)
-    self->flags |= AFFILE_CREATE_DIRS;
+    self->file_open_options.create_dirs = TRUE;
   if (self->time_reap == -1)
     self->time_reap = cfg->time_reap;
   
   file_perm_options_init(&self->file_perm_options, cfg);
   log_writer_options_init(&self->writer_options, cfg, 0);
-  log_template_options_init(&self->template_fname_options, cfg);
               
-  if ((self->flags & AFFILE_NO_EXPAND) == 0)
+  if (self->filename_is_a_template)
     {
       self->writer_hash = cfg_persist_config_fetch(cfg, affile_dd_format_persist_name(self));
       if (self->writer_hash)
@@ -449,11 +462,14 @@ affile_dd_init(LogPipe *s)
       if (self->single_writer)
         {
           affile_dw_set_owner(self->single_writer, self);
-          log_pipe_init(&self->single_writer->super, cfg);
+          if (!log_pipe_init(&self->single_writer->super))
+            {
+              log_pipe_unref(&self->single_writer->super);
+              return FALSE;
+            }
         }
     }
-  
-  
+
   return TRUE;
 }
 
@@ -543,16 +559,15 @@ static LogPipe *
 affile_dd_open_writer(gpointer args[])
 {
   AFFileDestDriver *self = args[0];
-  GlobalConfig *cfg = log_pipe_get_config(&self->super.super.super);
   AFFileDestWriter *next;
 
   main_loop_assert_main_thread();
-  if (self->flags & AFFILE_NO_EXPAND)
+  if (!self->filename_is_a_template)
     {
       if (!self->single_writer)
 	{
 	  next = affile_dw_new(self, self->filename_template->template);
-          if (next && log_pipe_init(&next->super, cfg))
+          if (next && log_pipe_init(&next->super))
 	    {
 	      log_pipe_ref(&next->super);
 	      g_static_mutex_lock(&self->lock);
@@ -588,7 +603,7 @@ affile_dd_open_writer(gpointer args[])
       if (!next)
 	{
 	  next = affile_dw_new(self, filename->str);
-          if (!log_pipe_init(&next->super, cfg))
+          if (!log_pipe_init(&next->super))
 	    {
 	      log_pipe_unref(&next->super);
 	      next = NULL;
@@ -623,7 +638,7 @@ affile_dd_queue(LogPipe *s, LogMessage *msg, const LogPathOptions *path_options,
   AFFileDestWriter *next;
   gpointer args[2] = { self, NULL };
 
-  if (self->flags & AFFILE_NO_EXPAND)
+  if (!self->filename_is_a_template)
     {
       /* no need to lock the check below, the worst case that happens is
        * that we go to the mainloop to return the same information, but this
@@ -650,7 +665,7 @@ affile_dd_queue(LogPipe *s, LogMessage *msg, const LogPathOptions *path_options,
       GString *filename;
 
       filename = g_string_sized_new(32);
-      log_template_format(self->filename_template, msg, &self->template_fname_options, LTZ_LOCAL, 0, NULL, filename);
+      log_template_format(self->filename_template, msg, &self->writer_options.template_options, LTZ_LOCAL, 0, NULL, filename);
 
       g_static_mutex_lock(&self->lock);
       if (self->writer_hash)
@@ -691,35 +706,50 @@ affile_dd_free(LogPipe *s)
   /* NOTE: this must be NULL as deinit has freed it, otherwise we'd have circular references */
   g_assert(self->single_writer == NULL && self->writer_hash == NULL);
 
-  log_template_options_destroy(&self->template_fname_options);
-  g_static_mutex_free(&self->lock);
   log_template_unref(self->filename_template);
   log_writer_options_destroy(&self->writer_options);
   log_dest_driver_free(s);
 }
 
-LogDriver *
-affile_dd_new(gchar *filename, guint32 flags)
+static AFFileDestDriver *
+affile_dd_new_instance(gchar *filename, GlobalConfig *cfg)
 {
   AFFileDestDriver *self = g_new0(AFFileDestDriver, 1);
 
-  log_dest_driver_init_instance(&self->super);
+  log_dest_driver_init_instance(&self->super, cfg);
   self->super.super.super.init = affile_dd_init;
   self->super.super.super.deinit = affile_dd_deinit;
   self->super.super.super.queue = affile_dd_queue;
   self->super.super.super.free_fn = affile_dd_free;
-  self->filename_template = log_template_new(configuration, NULL);
+  self->filename_template = log_template_new(cfg, NULL);
   log_template_compile(self->filename_template, filename, NULL);
-  self->flags = flags;
   log_writer_options_defaults(&self->writer_options);
   file_perm_options_defaults(&self->file_perm_options);
   self->writer_options.mark_mode = MM_NONE;
-  if (strchr(filename, '$') == NULL)
+  if (strchr(filename, '$') != NULL)
     {
-      self->flags |= AFFILE_NO_EXPAND;
+      self->filename_is_a_template = TRUE;
     }
   self->time_reap = -1;
-  log_template_options_defaults(&self->template_fname_options);
+  self->file_open_options.is_pipe = FALSE;
+  self->file_open_options.needs_privileges = FALSE;
+  self->file_open_options.open_flags = DEFAULT_DW_REOPEN_FLAGS;
   g_static_mutex_init(&self->lock);
+  return self;
+}
+
+LogDriver *
+affile_dd_new(gchar *filename, GlobalConfig *cfg)
+{
+  return &affile_dd_new_instance(filename, cfg)->super.super;
+}
+
+LogDriver *
+afpipe_dd_new(gchar *filename, GlobalConfig *cfg)
+{
+  AFFileDestDriver *self = affile_dd_new_instance(filename, cfg);
+  self->file_open_options.is_pipe = TRUE;
+  self->file_open_options.open_flags = DEFAULT_DW_REOPEN_FLAGS_PIPE;
+
   return &self->super.super;
 }
