@@ -27,9 +27,11 @@
 #include "logwriter.h"
 #include "children.h"
 #include "misc.h"
-#include "stats.h"
-#include "logproto-text-server.h"
-#include "logproto-text-client.h"
+#include "stats/stats-registry.h"
+#include "transport/transport-pipe.h"
+#include "logproto/logproto-text-server.h"
+#include "logproto/logproto-text-client.h"
+#include "poll-fd-events.h"
 
 #include <sys/types.h>
 #include <fcntl.h>
@@ -172,16 +174,23 @@ afprogram_sd_init(LogPipe *s)
       LogTransport *transport;
 
       transport = log_transport_pipe_new(fd);
-      self->reader = log_reader_new(log_proto_text_server_new(transport, &self->reader_options.proto_options.super));
-      log_reader_set_options(self->reader, s, &self->reader_options, 0, SCS_PROGRAM, self->super.super.id, self->cmdline->str);
+      self->reader = log_reader_new(s->cfg);
+      log_reader_reopen(self->reader, log_proto_text_server_new(transport, &self->reader_options.proto_options.super), poll_fd_events_new(fd));
+      log_reader_set_options(self->reader,
+                             s,
+                             &self->reader_options,
+                             STATS_LEVEL0,
+                             SCS_PROGRAM,
+                             self->super.super.id,
+                             self->cmdline->str);
     }
-  log_pipe_append(self->reader, &self->super.super.super);
-  if (!log_pipe_init(self->reader, NULL))
+  log_pipe_append((LogPipe *) self->reader, &self->super.super.super);
+  if (!log_pipe_init((LogPipe *) self->reader))
     { 
       msg_error("Error initializing program source, closing fd",
                 evt_tag_int("fd", fd),
                 NULL);
-      log_pipe_unref(self->reader);
+      log_pipe_unref((LogPipe *) self->reader);
       self->reader = NULL;
       close(fd);
       return FALSE;
@@ -197,8 +206,8 @@ afprogram_sd_deinit(LogPipe *s)
   afprogram_sd_kill_child(self);
   if (self->reader)
     {
-      log_pipe_deinit(self->reader);
-      log_pipe_unref(self->reader);
+      log_pipe_deinit((LogPipe *) self->reader);
+      log_pipe_unref((LogPipe *) self->reader);
       self->reader = NULL;
     }
 
@@ -219,7 +228,7 @@ afprogram_sd_free(LogPipe *s)
 }
 
 static void
-afprogram_sd_notify(LogPipe *s, LogPipe *sender, gint notify_code, gpointer user_data)
+afprogram_sd_notify(LogPipe *s, gint notify_code, gpointer user_data)
 {
   switch (notify_code)
     {
@@ -232,10 +241,10 @@ afprogram_sd_notify(LogPipe *s, LogPipe *sender, gint notify_code, gpointer user
 }
 
 LogDriver *
-afprogram_sd_new(gchar *cmdline)
+afprogram_sd_new(gchar *cmdline, GlobalConfig *cfg)
 {
   AFProgramSourceDriver *self = g_new0(AFProgramSourceDriver, 1);
-  log_src_driver_init_instance(&self->super);
+  log_src_driver_init_instance(&self->super, cfg);
   
   self->super.super.super.init = afprogram_sd_init;
   self->super.super.super.deinit = afprogram_sd_deinit;
@@ -267,11 +276,12 @@ afprogram_dd_kill_child(AFProgramDestDriver *self)
 {
   if (self->pid != -1)
     {
+      child_manager_unregister(self->pid);
       msg_verbose("Sending destination program a TERM signal",
                   evt_tag_str("cmdline", self->cmdline->str),
                   evt_tag_int("child_pid", self->pid),
                   NULL);
-      kill(-self->pid, SIGTERM);
+      killpg(getpgid(self->pid), SIGTERM);
       self->pid = -1;
     }
 }
@@ -327,13 +337,23 @@ afprogram_dd_init(LogPipe *s)
   log_writer_options_init(&self->writer_options, cfg, 0);
 
   if (!self->writer)
-    self->writer = log_writer_new(LW_FORMAT_FILE);
+    self->writer = log_writer_new(LW_FORMAT_FILE, s->cfg);
 
-  log_writer_set_options((LogWriter *) self->writer, s, &self->writer_options, 0, SCS_PROGRAM, self->super.super.id, self->cmdline->str);
+  log_writer_set_options(self->writer,
+                         s,
+                         &self->writer_options,
+                         STATS_LEVEL0,
+                         SCS_PROGRAM,
+                         self->super.super.id,
+                         self->cmdline->str);
   log_writer_set_queue(self->writer, log_dest_driver_acquire_queue(&self->super, afprogram_dd_format_persist_name(self)));
 
-  log_pipe_init(self->writer, NULL);
-  log_pipe_append(&self->super.super.super, self->writer);
+  if (!log_pipe_init((LogPipe *) self->writer))
+    {
+      log_pipe_unref((LogPipe *) self->writer);
+      return FALSE;
+    }
+  log_pipe_append(&self->super.super.super, (LogPipe *) self->writer);
 
   return afprogram_dd_reopen(self);
 }
@@ -346,8 +366,8 @@ afprogram_dd_deinit(LogPipe *s)
   afprogram_dd_kill_child(self);
   if (self->writer)
     {
-      log_pipe_deinit(self->writer);
-      log_pipe_unref(self->writer);
+      log_pipe_deinit((LogPipe *) self->writer);
+      log_pipe_unref((LogPipe *) self->writer);
       self->writer = NULL;
     }
 
@@ -361,14 +381,14 @@ afprogram_dd_free(LogPipe *s)
 {
   AFProgramDestDriver *self = (AFProgramDestDriver *) s;
 
-  log_pipe_unref(self->writer);  
+  log_pipe_unref((LogPipe *) self->writer);
   g_string_free(self->cmdline, TRUE);
   log_writer_options_destroy(&self->writer_options);
   log_dest_driver_free(s);
 }
 
 static void
-afprogram_dd_notify(LogPipe *s, LogPipe *sender, gint notify_code, gpointer user_data)
+afprogram_dd_notify(LogPipe *s, gint notify_code, gpointer user_data)
 {
   AFProgramDestDriver *self = (AFProgramDestDriver *) s;
 
@@ -382,10 +402,10 @@ afprogram_dd_notify(LogPipe *s, LogPipe *sender, gint notify_code, gpointer user
 }
 
 LogDriver *
-afprogram_dd_new(gchar *cmdline)
+afprogram_dd_new(gchar *cmdline, GlobalConfig *cfg)
 {
   AFProgramDestDriver *self = g_new0(AFProgramDestDriver, 1);
-  log_dest_driver_init_instance(&self->super);
+  log_dest_driver_init_instance(&self->super, cfg);
   
   self->super.super.super.init = afprogram_dd_init;
   self->super.super.super.deinit = afprogram_dd_deinit;
