@@ -30,8 +30,8 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-#ifndef YY_AFSMTP_AFSMTP_GRAMMAR_H_INCLUDED
-# define YY_AFSMTP_AFSMTP_GRAMMAR_H_INCLUDED
+#ifndef YY_AFSMTP_MODULES_AFSMTP_AFSMTP_GRAMMAR_H_INCLUDED
+# define YY_AFSMTP_MODULES_AFSMTP_AFSMTP_GRAMMAR_H_INCLUDED
 /* Debug traces.  */
 #ifndef YYDEBUG
 # define YYDEBUG 0
@@ -40,13 +40,13 @@
 extern int afsmtp_debug;
 #endif
 /* "%code requires" blocks.  */
-#line 24 "afsmtp-grammar.y" /* yacc.c:1909  */
+#line 24 "modules/afsmtp/afsmtp-grammar.y" /* yacc.c:1909  */
 
 
 #include "afsmtp-parser.h"
 
 
-#line 50 "afsmtp-grammar.h" /* yacc.c:1909  */
+#line 50 "modules/afsmtp/afsmtp-grammar.h" /* yacc.c:1909  */
 
 /* Token type.  */
 #ifndef YYTOKENTYPE
@@ -102,13 +102,14 @@ extern int afsmtp_debug;
     KW_MARK_FREQ = 10071,
     KW_STATS_FREQ = 10072,
     KW_STATS_LEVEL = 10073,
-    KW_FLUSH_LINES = 10074,
-    KW_SUPPRESS = 10075,
-    KW_FLUSH_TIMEOUT = 10076,
-    KW_LOG_MSG_SIZE = 10077,
-    KW_FILE_TEMPLATE = 10078,
-    KW_PROTO_TEMPLATE = 10079,
-    KW_MARK_MODE = 10080,
+    KW_STATS_LIFETIME = 10074,
+    KW_FLUSH_LINES = 10075,
+    KW_SUPPRESS = 10076,
+    KW_FLUSH_TIMEOUT = 10077,
+    KW_LOG_MSG_SIZE = 10078,
+    KW_FILE_TEMPLATE = 10079,
+    KW_PROTO_TEMPLATE = 10080,
+    KW_MARK_MODE = 10081,
     KW_CHAIN_HOSTNAMES = 10090,
     KW_NORMALIZE_HOSTNAMES = 10091,
     KW_KEEP_HOSTNAME = 10092,
@@ -117,12 +118,14 @@ extern int afsmtp_debug;
     KW_KEEP_TIMESTAMP = 10100,
     KW_USE_DNS = 10110,
     KW_USE_FQDN = 10111,
+    KW_CUSTOM_DOMAIN = 10112,
     KW_DNS_CACHE = 10120,
     KW_DNS_CACHE_SIZE = 10121,
     KW_DNS_CACHE_EXPIRE = 10130,
     KW_DNS_CACHE_EXPIRE_FAILED = 10131,
     KW_DNS_CACHE_HOSTS = 10132,
     KW_PERSIST_ONLY = 10140,
+    KW_USE_RCPTID = 10141,
     KW_TZ_CONVERT = 10150,
     KW_TS_FORMAT = 10151,
     KW_FRAC_DIGITS = 10152,
@@ -193,17 +196,19 @@ extern int afsmtp_debug;
     KW_SHIFT = 10506,
     KW_REKEY = 10507,
     KW_ADD_PREFIX = 10508,
-    KW_REPLACE = 10509,
-    KW_SMTP = 10511,
-    KW_SUBJECT = 10512,
-    KW_FROM = 10513,
-    KW_TO = 10514,
-    KW_BODY = 10515,
-    KW_HEADER = 10516,
-    KW_CC = 10517,
-    KW_BCC = 10518,
-    KW_SENDER = 10519,
-    KW_REPLY_TO = 10520
+    KW_REPLACE_PREFIX = 10509,
+    KW_ON_ERROR = 10510,
+    KW_RETRIES = 10511,
+    KW_SMTP = 10513,
+    KW_SUBJECT = 10514,
+    KW_FROM = 10515,
+    KW_TO = 10516,
+    KW_BODY = 10517,
+    KW_HEADER = 10518,
+    KW_CC = 10519,
+    KW_BCC = 10520,
+    KW_SENDER = 10521,
+    KW_REPLY_TO = 10522
   };
 #endif
 /* Tokens.  */
@@ -256,13 +261,14 @@ extern int afsmtp_debug;
 #define KW_MARK_FREQ 10071
 #define KW_STATS_FREQ 10072
 #define KW_STATS_LEVEL 10073
-#define KW_FLUSH_LINES 10074
-#define KW_SUPPRESS 10075
-#define KW_FLUSH_TIMEOUT 10076
-#define KW_LOG_MSG_SIZE 10077
-#define KW_FILE_TEMPLATE 10078
-#define KW_PROTO_TEMPLATE 10079
-#define KW_MARK_MODE 10080
+#define KW_STATS_LIFETIME 10074
+#define KW_FLUSH_LINES 10075
+#define KW_SUPPRESS 10076
+#define KW_FLUSH_TIMEOUT 10077
+#define KW_LOG_MSG_SIZE 10078
+#define KW_FILE_TEMPLATE 10079
+#define KW_PROTO_TEMPLATE 10080
+#define KW_MARK_MODE 10081
 #define KW_CHAIN_HOSTNAMES 10090
 #define KW_NORMALIZE_HOSTNAMES 10091
 #define KW_KEEP_HOSTNAME 10092
@@ -271,12 +277,14 @@ extern int afsmtp_debug;
 #define KW_KEEP_TIMESTAMP 10100
 #define KW_USE_DNS 10110
 #define KW_USE_FQDN 10111
+#define KW_CUSTOM_DOMAIN 10112
 #define KW_DNS_CACHE 10120
 #define KW_DNS_CACHE_SIZE 10121
 #define KW_DNS_CACHE_EXPIRE 10130
 #define KW_DNS_CACHE_EXPIRE_FAILED 10131
 #define KW_DNS_CACHE_HOSTS 10132
 #define KW_PERSIST_ONLY 10140
+#define KW_USE_RCPTID 10141
 #define KW_TZ_CONVERT 10150
 #define KW_TS_FORMAT 10151
 #define KW_FRAC_DIGITS 10152
@@ -347,17 +355,19 @@ extern int afsmtp_debug;
 #define KW_SHIFT 10506
 #define KW_REKEY 10507
 #define KW_ADD_PREFIX 10508
-#define KW_REPLACE 10509
-#define KW_SMTP 10511
-#define KW_SUBJECT 10512
-#define KW_FROM 10513
-#define KW_TO 10514
-#define KW_BODY 10515
-#define KW_HEADER 10516
-#define KW_CC 10517
-#define KW_BCC 10518
-#define KW_SENDER 10519
-#define KW_REPLY_TO 10520
+#define KW_REPLACE_PREFIX 10509
+#define KW_ON_ERROR 10510
+#define KW_RETRIES 10511
+#define KW_SMTP 10513
+#define KW_SUBJECT 10514
+#define KW_FROM 10515
+#define KW_TO 10516
+#define KW_BODY 10517
+#define KW_HEADER 10518
+#define KW_CC 10519
+#define KW_BCC 10520
+#define KW_SENDER 10521
+#define KW_REPLY_TO 10522
 
 /* Value type.  */
 
@@ -379,4 +389,4 @@ struct YYLTYPE
 
 int afsmtp_parse (CfgLexer *lexer, LogDriver **instance, gpointer arg);
 
-#endif /* !YY_AFSMTP_AFSMTP_GRAMMAR_H_INCLUDED  */
+#endif /* !YY_AFSMTP_MODULES_AFSMTP_AFSMTP_GRAMMAR_H_INCLUDED  */
