@@ -30,8 +30,8 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-#ifndef YY_AFSOCKET_AFSOCKET_GRAMMAR_H_INCLUDED
-# define YY_AFSOCKET_AFSOCKET_GRAMMAR_H_INCLUDED
+#ifndef YY_AFSOCKET_MODULES_AFSOCKET_AFSOCKET_GRAMMAR_H_INCLUDED
+# define YY_AFSOCKET_MODULES_AFSOCKET_AFSOCKET_GRAMMAR_H_INCLUDED
 /* Debug traces.  */
 #ifndef YYDEBUG
 # define YYDEBUG 0
@@ -54,31 +54,32 @@ extern int afsocket_debug;
     KW_NETWORK = 20007,
     KW_TRANSPORT = 20008,
     KW_IP_PROTOCOL = 20009,
-    KW_IP_TTL = 20010,
-    KW_SO_BROADCAST = 20011,
-    KW_IP_TOS = 20012,
-    KW_SO_SNDBUF = 20013,
-    KW_SO_RCVBUF = 20014,
-    KW_SO_KEEPALIVE = 20015,
-    KW_TCP_KEEPALIVE_TIME = 20016,
-    KW_TCP_KEEPALIVE_PROBES = 20017,
-    KW_TCP_KEEPALIVE_INTVL = 20018,
-    KW_SPOOF_SOURCE = 20019,
-    KW_KEEP_ALIVE = 20020,
-    KW_MAX_CONNECTIONS = 20021,
-    KW_LOCALIP = 20022,
-    KW_IP = 20023,
-    KW_LOCALPORT = 20024,
-    KW_DESTPORT = 20025,
-    KW_TLS = 20026,
-    KW_PEER_VERIFY = 20027,
-    KW_KEY_FILE = 20028,
-    KW_CERT_FILE = 20029,
-    KW_CA_DIR = 20030,
-    KW_CRL_DIR = 20031,
-    KW_TRUSTED_KEYS = 20032,
-    KW_TRUSTED_DN = 20033,
-    KW_CIPHER_SUITE = 20034,
+    KW_SYSTEMD_SYSLOG = 20010,
+    KW_IP_TTL = 20011,
+    KW_SO_BROADCAST = 20012,
+    KW_IP_TOS = 20013,
+    KW_SO_SNDBUF = 20014,
+    KW_SO_RCVBUF = 20015,
+    KW_SO_KEEPALIVE = 20016,
+    KW_TCP_KEEPALIVE_TIME = 20017,
+    KW_TCP_KEEPALIVE_PROBES = 20018,
+    KW_TCP_KEEPALIVE_INTVL = 20019,
+    KW_SPOOF_SOURCE = 20020,
+    KW_KEEP_ALIVE = 20021,
+    KW_MAX_CONNECTIONS = 20022,
+    KW_LOCALIP = 20023,
+    KW_IP = 20024,
+    KW_LOCALPORT = 20025,
+    KW_DESTPORT = 20026,
+    KW_TLS = 20027,
+    KW_PEER_VERIFY = 20028,
+    KW_KEY_FILE = 20029,
+    KW_CERT_FILE = 20030,
+    KW_CA_DIR = 20031,
+    KW_CRL_DIR = 20032,
+    KW_TRUSTED_KEYS = 20033,
+    KW_TRUSTED_DN = 20034,
+    KW_CIPHER_SUITE = 20035,
     LL_CONTEXT_ROOT = 1,
     LL_CONTEXT_DESTINATION = 2,
     LL_CONTEXT_SOURCE = 3,
@@ -128,13 +129,14 @@ extern int afsocket_debug;
     KW_MARK_FREQ = 10071,
     KW_STATS_FREQ = 10072,
     KW_STATS_LEVEL = 10073,
-    KW_FLUSH_LINES = 10074,
-    KW_SUPPRESS = 10075,
-    KW_FLUSH_TIMEOUT = 10076,
-    KW_LOG_MSG_SIZE = 10077,
-    KW_FILE_TEMPLATE = 10078,
-    KW_PROTO_TEMPLATE = 10079,
-    KW_MARK_MODE = 10080,
+    KW_STATS_LIFETIME = 10074,
+    KW_FLUSH_LINES = 10075,
+    KW_SUPPRESS = 10076,
+    KW_FLUSH_TIMEOUT = 10077,
+    KW_LOG_MSG_SIZE = 10078,
+    KW_FILE_TEMPLATE = 10079,
+    KW_PROTO_TEMPLATE = 10080,
+    KW_MARK_MODE = 10081,
     KW_CHAIN_HOSTNAMES = 10090,
     KW_NORMALIZE_HOSTNAMES = 10091,
     KW_KEEP_HOSTNAME = 10092,
@@ -143,12 +145,14 @@ extern int afsocket_debug;
     KW_KEEP_TIMESTAMP = 10100,
     KW_USE_DNS = 10110,
     KW_USE_FQDN = 10111,
+    KW_CUSTOM_DOMAIN = 10112,
     KW_DNS_CACHE = 10120,
     KW_DNS_CACHE_SIZE = 10121,
     KW_DNS_CACHE_EXPIRE = 10130,
     KW_DNS_CACHE_EXPIRE_FAILED = 10131,
     KW_DNS_CACHE_HOSTS = 10132,
     KW_PERSIST_ONLY = 10140,
+    KW_USE_RCPTID = 10141,
     KW_TZ_CONVERT = 10150,
     KW_TS_FORMAT = 10151,
     KW_FRAC_DIGITS = 10152,
@@ -219,7 +223,9 @@ extern int afsocket_debug;
     KW_SHIFT = 10506,
     KW_REKEY = 10507,
     KW_ADD_PREFIX = 10508,
-    KW_REPLACE = 10509
+    KW_REPLACE_PREFIX = 10509,
+    KW_ON_ERROR = 10510,
+    KW_RETRIES = 10511
   };
 #endif
 /* Tokens.  */
@@ -232,31 +238,32 @@ extern int afsocket_debug;
 #define KW_NETWORK 20007
 #define KW_TRANSPORT 20008
 #define KW_IP_PROTOCOL 20009
-#define KW_IP_TTL 20010
-#define KW_SO_BROADCAST 20011
-#define KW_IP_TOS 20012
-#define KW_SO_SNDBUF 20013
-#define KW_SO_RCVBUF 20014
-#define KW_SO_KEEPALIVE 20015
-#define KW_TCP_KEEPALIVE_TIME 20016
-#define KW_TCP_KEEPALIVE_PROBES 20017
-#define KW_TCP_KEEPALIVE_INTVL 20018
-#define KW_SPOOF_SOURCE 20019
-#define KW_KEEP_ALIVE 20020
-#define KW_MAX_CONNECTIONS 20021
-#define KW_LOCALIP 20022
-#define KW_IP 20023
-#define KW_LOCALPORT 20024
-#define KW_DESTPORT 20025
-#define KW_TLS 20026
-#define KW_PEER_VERIFY 20027
-#define KW_KEY_FILE 20028
-#define KW_CERT_FILE 20029
-#define KW_CA_DIR 20030
-#define KW_CRL_DIR 20031
-#define KW_TRUSTED_KEYS 20032
-#define KW_TRUSTED_DN 20033
-#define KW_CIPHER_SUITE 20034
+#define KW_SYSTEMD_SYSLOG 20010
+#define KW_IP_TTL 20011
+#define KW_SO_BROADCAST 20012
+#define KW_IP_TOS 20013
+#define KW_SO_SNDBUF 20014
+#define KW_SO_RCVBUF 20015
+#define KW_SO_KEEPALIVE 20016
+#define KW_TCP_KEEPALIVE_TIME 20017
+#define KW_TCP_KEEPALIVE_PROBES 20018
+#define KW_TCP_KEEPALIVE_INTVL 20019
+#define KW_SPOOF_SOURCE 20020
+#define KW_KEEP_ALIVE 20021
+#define KW_MAX_CONNECTIONS 20022
+#define KW_LOCALIP 20023
+#define KW_IP 20024
+#define KW_LOCALPORT 20025
+#define KW_DESTPORT 20026
+#define KW_TLS 20027
+#define KW_PEER_VERIFY 20028
+#define KW_KEY_FILE 20029
+#define KW_CERT_FILE 20030
+#define KW_CA_DIR 20031
+#define KW_CRL_DIR 20032
+#define KW_TRUSTED_KEYS 20033
+#define KW_TRUSTED_DN 20034
+#define KW_CIPHER_SUITE 20035
 #define LL_CONTEXT_ROOT 1
 #define LL_CONTEXT_DESTINATION 2
 #define LL_CONTEXT_SOURCE 3
@@ -306,13 +313,14 @@ extern int afsocket_debug;
 #define KW_MARK_FREQ 10071
 #define KW_STATS_FREQ 10072
 #define KW_STATS_LEVEL 10073
-#define KW_FLUSH_LINES 10074
-#define KW_SUPPRESS 10075
-#define KW_FLUSH_TIMEOUT 10076
-#define KW_LOG_MSG_SIZE 10077
-#define KW_FILE_TEMPLATE 10078
-#define KW_PROTO_TEMPLATE 10079
-#define KW_MARK_MODE 10080
+#define KW_STATS_LIFETIME 10074
+#define KW_FLUSH_LINES 10075
+#define KW_SUPPRESS 10076
+#define KW_FLUSH_TIMEOUT 10077
+#define KW_LOG_MSG_SIZE 10078
+#define KW_FILE_TEMPLATE 10079
+#define KW_PROTO_TEMPLATE 10080
+#define KW_MARK_MODE 10081
 #define KW_CHAIN_HOSTNAMES 10090
 #define KW_NORMALIZE_HOSTNAMES 10091
 #define KW_KEEP_HOSTNAME 10092
@@ -321,12 +329,14 @@ extern int afsocket_debug;
 #define KW_KEEP_TIMESTAMP 10100
 #define KW_USE_DNS 10110
 #define KW_USE_FQDN 10111
+#define KW_CUSTOM_DOMAIN 10112
 #define KW_DNS_CACHE 10120
 #define KW_DNS_CACHE_SIZE 10121
 #define KW_DNS_CACHE_EXPIRE 10130
 #define KW_DNS_CACHE_EXPIRE_FAILED 10131
 #define KW_DNS_CACHE_HOSTS 10132
 #define KW_PERSIST_ONLY 10140
+#define KW_USE_RCPTID 10141
 #define KW_TZ_CONVERT 10150
 #define KW_TS_FORMAT 10151
 #define KW_FRAC_DIGITS 10152
@@ -397,7 +407,9 @@ extern int afsocket_debug;
 #define KW_SHIFT 10506
 #define KW_REKEY 10507
 #define KW_ADD_PREFIX 10508
-#define KW_REPLACE 10509
+#define KW_REPLACE_PREFIX 10509
+#define KW_ON_ERROR 10510
+#define KW_RETRIES 10511
 
 /* Value type.  */
 
@@ -419,4 +431,4 @@ struct YYLTYPE
 
 int afsocket_parse (CfgLexer *lexer, LogDriver **instance, gpointer arg);
 
-#endif /* !YY_AFSOCKET_AFSOCKET_GRAMMAR_H_INCLUDED  */
+#endif /* !YY_AFSOCKET_MODULES_AFSOCKET_AFSOCKET_GRAMMAR_H_INCLUDED  */
