@@ -24,10 +24,10 @@
 #include "patterndb.h"
 #include "logmsg.h"
 #include "tags.h"
-#include "templates.h"
-#include "compat.h"
+#include "template/templates.h"
 #include "misc.h"
-#include "filter-expr-parser.h"
+#include "filter/filter-expr-parser.h"
+#include "logpipe.h"
 #include "patterndb-int.h"
 
 #include <string.h>
@@ -37,6 +37,7 @@
 
 static NVHandle class_handle = 0;
 static NVHandle rule_id_handle = 0;
+static NVHandle context_id_handle = 0;
 static LogTagId system_tag;
 static LogTagId unknown_tag;
 
@@ -301,9 +302,10 @@ pdb_message_apply(PDBMessage *self, PDBContext *context, LogMessage *msg, GStrin
                                            context ? (LogMessage **) context->messages->pdata : &msg,
                                            context ? context->messages->len : 1,
                                            NULL, LTZ_LOCAL, 0, context ? context->key.session_id : NULL, buffer);
-          log_msg_set_value(msg,
-                            log_msg_get_value_handle(((LogTemplate *) g_ptr_array_index(self->values, i))->name),
-                            buffer->str, buffer->len);
+          log_msg_set_value_by_name(msg,
+                                    ((LogTemplate *) g_ptr_array_index(self->values, i))->name,
+                                    buffer->str,
+                                    buffer->len);
         }
     }
 
@@ -557,8 +559,10 @@ pdb_rule_run_actions(PDBRule *self, gint trigger, PatternDB *db, PDBContext *con
           LogMessage *genmsg;
 
           if ((!action->condition ||
-               (!context || filter_expr_eval_with_context(action->condition, (LogMessage **) context->messages->pdata, context->messages->len))) &&
-              pdb_rule_check_rate_limit(self, db, action, msg, buffer))
+	      (context ? 
+		(filter_expr_eval_with_context(action->condition, (LogMessage **) context->messages->pdata, context->messages->len)) : 
+		filter_expr_eval(action->condition, msg) ) 
+	      && pdb_rule_check_rate_limit(self, db, action, msg, buffer)))
             {
               switch (action->content_type)
                 {
@@ -857,9 +861,13 @@ pdb_loader_start_element(GMarkupParseContext *context, const gchar *element_name
       if (attribute_names[0] && g_str_equal(attribute_names[0], "name"))
         state->test_value_name = g_strdup(attribute_values[0]);
       else
-        msg_error("No name is specified for test_value",
-                  evt_tag_str("rule_id", state->current_rule->rule_id),
-                  NULL);
+        {
+          msg_error("No name is specified for test_value",
+                    evt_tag_str("rule_id", state->current_rule->rule_id),
+                    NULL);
+          *error = g_error_new(1, 0, "<test_value> misses name attribute");
+          return;
+        }
     }
   else if (strcmp(element_name, "rule") == 0)
     {
@@ -915,7 +923,11 @@ pdb_loader_start_element(GMarkupParseContext *context, const gchar *element_name
       if (attribute_names[0] && g_str_equal(attribute_names[0], "name"))
         state->value_name = g_strdup(attribute_values[0]);
       else
-        msg_error("No name is specified for value", evt_tag_str("rule_id", state->current_rule->rule_id), NULL);
+        {
+          msg_error("No name is specified for value", evt_tag_str("rule_id", state->current_rule->rule_id), NULL);
+          *error = g_error_new(1, 0, "<value> misses name attribute");
+          return;
+        }
     }
   else if (strcmp(element_name, "patterndb") == 0)
     {
@@ -1178,8 +1190,9 @@ pdb_loader_text(GMarkupParseContext *context, const gchar *text, gsize text_len,
             evt_tag_str("name", state->value_name),
             evt_tag_str("value", text),
             evt_tag_str("error", err->message), NULL);
-          g_clear_error(&err);
           log_template_unref(value);
+          g_propagate_error(error, err);
+          return;
         }
       else
         g_ptr_array_add(state->current_message->values, value);
@@ -1489,6 +1502,14 @@ pattern_db_timer_tick(PatternDB *self)
       self->last_tick = now;
       g_time_val_add(&self->last_tick, -(diff - diff_sec * 1e6));
     }
+  else if (diff < 0)
+    {
+      /* time moving backwards, this can only happen if the computer's time
+       * is changed.  We don't update patterndb's idea of the time now, wait
+       * another tick instead to update that instead.
+       */
+      self->last_tick = now;
+    }
   g_static_rw_lock_writer_unlock(&self->lock);
 }
 
@@ -1602,6 +1623,7 @@ pattern_db_process(PatternDB *self, PDBInput *input)
           PDBStateKey key;
 
           log_template_format(rule->context_id_template, msg, NULL, LTZ_LOCAL, 0, NULL, buffer);
+          log_msg_set_value(msg, context_id_handle, buffer->str, -1);
 
           pdb_state_key_setup(&key, PSK_CONTEXT, rule, msg, buffer->str);
           context = g_hash_table_lookup(self->state, &key);
@@ -1707,6 +1729,7 @@ pattern_db_global_init(void)
 {
   class_handle = log_msg_get_value_handle(".classifier.class");
   rule_id_handle = log_msg_get_value_handle(".classifier.rule_id");
+  context_id_handle = log_msg_get_value_handle(".classifier.context_id");
   system_tag = log_tags_get_by_name(".classifier.system");
   unknown_tag = log_tags_get_by_name(".classifier.unknown");
 }
