@@ -2,9 +2,10 @@
 #include "tags.h"
 #include "logmsg.h"
 #include "messages.h"
-#include "filter.h"
+#include "filter/filter-expr.h"
 #include "patterndb.h"
 #include "plugin.h"
+#include "cfg.h"
 #include "patterndb-int.h"
 
 #include <stdio.h>
@@ -73,6 +74,7 @@ clean_pattern_db(void)
       g_ptr_array_foreach(messages, (GFunc) log_msg_unref, NULL);
       g_ptr_array_free(messages, TRUE);
     }
+  messages = NULL;
   pattern_db_free(patterndb);
   patterndb = NULL;
 
@@ -132,6 +134,23 @@ gchar *pdb_ruletest_skeleton = "<patterndb version='3' pub_date='2010-02-22'>\
     <pattern>pattern12a</pattern>\
    </patterns>\
   </rule>\
+  <rule provider='test' id='11' class='system'>\
+   <patterns>\
+    <pattern>contextlesstest @STRING:field:@</pattern>\
+   </patterns>\
+   <actions>\
+     <action condition='\"${field}\" == \"value1\"'>\
+       <message>\
+         <value name='MESSAGE'>message1</value>\
+       </message>\
+     </action>\
+     <action condition='\"${field}\" == \"value2\"'>\
+       <message>\
+         <value name='MESSAGE'>message2</value>\
+       </message>\
+     </action>\
+   </actions>\
+  </rule>\ 
  </ruleset>\
 </patterndb>";
 
@@ -160,7 +179,7 @@ test_rule_value_without_clean(const gchar *program, const gchar *pattern,
   log_msg_set_value(msg, LM_V_PID, MYPID, strlen(MYPID));
 
   result = pattern_db_process(patterndb, PDB_INPUT_WRAP_MESSAGE(&input, msg));
-  val = log_msg_get_value(msg, log_msg_get_value_handle(name), &len);
+  val = log_msg_get_value_by_name(msg, name, &len);
   if (value)
     found = strcmp(val, value) == 0;
 
@@ -224,11 +243,12 @@ test_rule_action_message_value(const gchar *pattern, gint timeout, gint ndx, con
       goto exit;
     }
 
-  val = log_msg_get_value((LogMessage *) g_ptr_array_index(messages, ndx), log_msg_get_value_handle(name), &len);
+  val = log_msg_get_value_by_name((LogMessage *) g_ptr_array_index(messages, ndx), name, &len);
   if (value)
     found = strcmp(val, value) == 0;
 
-  if (!!value ^ (len > 0))
+  if (((value == NULL) && (len != NULL)) || 
+      ((value != NULL) && ( (len == NULL) || (!found) )))
     test_fail("Value '%s' is %smatching for pattern '%s' (%d) index %d\n", name, found ? "" : "not ", pattern, !!result, ndx);
 
 exit:
@@ -298,6 +318,7 @@ test_patterndb_rule(void)
   test_rule_value("pattern11", "n11-2", "v11-2");
   test_rule_value("pattern11", "n11-3", NULL);
   test_rule_value("pattern11", "context-id", "999");
+  test_rule_value("pattern11", ".classifier.context_id", "999");
   test_rule_value("pattern11a", "n11-1", "v11-1");
   test_rule_value("pattern11a", "n11-2", "v11-2");
   test_rule_value("pattern11a", "n11-3", NULL);
@@ -320,6 +341,9 @@ test_patterndb_rule(void)
   test_rule_action_message_tag("pattern11", 60, 2, "tag11-3", FALSE);
   test_rule_action_message_tag("pattern11", 60, 2, "tag11-4", TRUE);
 
+  test_rule_action_message_value("contextlesstest value1", 0, 1, "MESSAGE",  "message1");
+  test_rule_action_message_value("contextlesstest value2", 0, 1, "MESSAGE",  "message2");
+
   clean_pattern_db();
 }
 
@@ -686,8 +710,8 @@ test_patterndb_context_length()
 {
   create_pattern_db(pdb_msg_count_skeleton);
 
-  test_rule_action_message_value("pattern13", 0, 1, "CONTEXT_LENGTH", "1");
-  test_rule_action_message_value("pattern14", 0, 1, "CONTEXT_LENGTH", "1");
+  test_rule_action_message_value("pattern13", 0, 1, "CONTEXT_LENGTH", "2");
+  test_rule_action_message_value("pattern14", 0, 1, "CONTEXT_LENGTH", "2");
 
   test_rule_value_without_clean("prog2", "pattern15-a", "p15", "-a");
   test_rule_action_message_value("pattern15-b", 0, 2, "fired", "true");
