@@ -1,7 +1,7 @@
 /*
- * Copyright (c) 2011-2012 BalaBit IT Ltd, Budapest, Hungary
+ * Copyright (c) 2011-2014 BalaBit IT Ltd, Budapest, Hungary
  * Copyright (c) 2011 Balint Kovacs <blint@balabit.hu>
- * Copyright (c) 2011-2012 Gergely Nagy <algernon@balabit.hu>
+ * Copyright (c) 2011-2014 Gergely Nagy <algernon@balabit.hu>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -23,12 +23,13 @@
  */
 
 #include "plugin.h"
-#include "templates.h"
-#include "filter.h"
-#include "filter-expr-parser.h"
+#include "template/templates.h"
+#include "filter/filter-expr.h"
+#include "filter/filter-expr-parser.h"
 #include "cfg.h"
 #include "value-pairs.h"
 #include "vptransform.h"
+#include "syslog-ng.h"
 
 typedef struct _TFJsonState
 {
@@ -50,7 +51,7 @@ tf_json_prepare(LogTemplateFunction *self, gpointer s, LogTemplate *parent,
 
   /* Always replace a leading dot with an underscore. */
   vpts = value_pairs_transform_set_new(".*");
-  value_pairs_transform_set_add_func(vpts, value_pairs_new_transform_replace(".", "_"));
+  value_pairs_transform_set_add_func(vpts, value_pairs_new_transform_replace_prefix(".", "_"));
   value_pairs_add_transforms(state->vp, vpts);
 
   return TRUE;
@@ -60,6 +61,7 @@ typedef struct
 {
   gboolean need_comma;
   GString *buffer;
+  const LogTemplateOptions *template_options;
 } json_state_t;
 
 static inline void
@@ -163,36 +165,104 @@ tf_json_obj_end(const gchar *name,
 }
 
 static gboolean
-tf_json_value(const gchar *name, const gchar *prefix, const gchar *value,
-              gpointer *prefix_data, gpointer user_data)
+tf_json_append_value(const gchar *name, const gchar *value,
+                     json_state_t *state, gboolean quoted)
 {
-  json_state_t *state = (json_state_t *)user_data;
-
   if (state->need_comma)
     g_string_append_c(state->buffer, ',');
 
   g_string_append_c(state->buffer, '"');
   g_string_append_escaped(state->buffer, name);
-  g_string_append(state->buffer, "\":\"");
+
+  if (quoted)
+    g_string_append(state->buffer, "\":\"");
+  else
+    g_string_append(state->buffer, "\":");
+
   g_string_append_escaped(state->buffer, value);
-  g_string_append_c(state->buffer, '"');
+
+  if (quoted)
+    g_string_append_c(state->buffer, '"');
+
+  return TRUE;
+}
+
+static gboolean
+tf_json_value(const gchar *name, const gchar *prefix,
+              TypeHint type, const gchar *value,
+              gpointer *prefix_data, gpointer user_data)
+{
+  json_state_t *state = (json_state_t *)user_data;
+  gint on_error = state->template_options->on_error;
+
+  switch (type)
+    {
+    case TYPE_HINT_STRING:
+    case TYPE_HINT_DATETIME:
+    default:
+      tf_json_append_value(name, value, state, TRUE);
+      break;
+    case TYPE_HINT_LITERAL:
+      tf_json_append_value(name, value, state, FALSE);
+      break;
+    case TYPE_HINT_INT32:
+    case TYPE_HINT_INT64:
+    case TYPE_HINT_DOUBLE:
+    case TYPE_HINT_BOOLEAN:
+      {
+        gint32 i32;
+        gint64 i64;
+        gdouble d;
+        gboolean b;
+        gboolean r = FALSE, fail = FALSE;
+        const gchar *v = value;
+
+        if (type == TYPE_HINT_INT32 &&
+            (fail = !type_cast_to_int32(value, &i32 , NULL)) == TRUE)
+          r = type_cast_drop_helper(on_error, value, "int32");
+        else if (type == TYPE_HINT_INT64 &&
+            (fail = !type_cast_to_int64(value, &i64 , NULL)) == TRUE)
+          r = type_cast_drop_helper(on_error, value, "int64");
+        else if (type == TYPE_HINT_DOUBLE &&
+            (fail = !type_cast_to_double(value, &d, NULL)) == TRUE)
+          r = type_cast_drop_helper(on_error, value, "double");
+        else if (type == TYPE_HINT_BOOLEAN)
+          {
+            if ((fail = !type_cast_to_boolean(value, &b , NULL)) == TRUE)
+              r = type_cast_drop_helper(on_error, value, "boolean");
+            else
+              v = b ? "true" : "false";
+          }
+
+        if (fail &&
+            !(on_error & ON_ERROR_FALLBACK_TO_STRING))
+          return r;
+
+        tf_json_append_value(name, v, state, fail);
+        break;
+      }
+    }
 
   state->need_comma = TRUE;
 
   return FALSE;
 }
 
-static void
-tf_json_append(GString *result, ValuePairs *vp, LogMessage *msg, gint32 seq_num)
+static gboolean
+tf_json_append(GString *result, ValuePairs *vp, LogMessage *msg,
+               const LogTemplateOptions *template_options, gint32 seq_num, gint time_zone_mode)
 {
   json_state_t state;
 
   state.need_comma = FALSE;
   state.buffer = result;
+  state.template_options = template_options;
 
-  value_pairs_walk(vp,
-                   tf_json_obj_start, tf_json_value, tf_json_obj_end,
-                   msg, seq_num, &state);
+  return value_pairs_walk(vp,
+                          tf_json_obj_start, tf_json_value, tf_json_obj_end,
+                          msg, seq_num, time_zone_mode,
+                          template_options,
+                          &state);
 }
 
 static void
@@ -201,9 +271,14 @@ tf_json_call(LogTemplateFunction *self, gpointer s,
 {
   TFJsonState *state = (TFJsonState *)s;
   gint i;
+  gboolean r = TRUE;
+  gsize orig_size = result->len;
 
   for (i = 0; i < args->num_messages; i++)
-    tf_json_append(result, state->vp, args->messages[i], args->seq_num);
+    r &= tf_json_append(result, state->vp, args->messages[i], args->opts, args->seq_num, args->tz);
+
+  if (!r && (args->opts->on_error & ON_ERROR_DROP_MESSAGE))
+    g_string_set_size(result, orig_size);
 }
 
 static void
@@ -212,7 +287,7 @@ tf_json_free_state(gpointer s)
   TFJsonState *state = (TFJsonState *)s;
 
   if (state->vp)
-    value_pairs_free(state->vp);
+    value_pairs_unref(state->vp);
   tf_simple_func_free_state(&state->super);
 }
 
