@@ -245,6 +245,7 @@ log_msg_parse_date(LogMessage *self, const guchar **data, gint *length, guint pa
     {
       /* RFC3339 timestamp, expected format: YYYY-MM-DDTHH:MM:SS[.frac]<+/->ZZ:ZZ */
       gint hours, mins;
+      time_t now_tv_sec = (time_t)now.tv_sec;
 
       self->timestamps[LM_TS_STAMP].tv_usec = 0;
 
@@ -253,7 +254,7 @@ log_msg_parse_date(LogMessage *self, const guchar **data, gint *length, guint pa
        * not exist on all platforms and 0 initializing it causes trouble on
        * time-zone barriers */
 
-      cached_localtime(&now.tv_sec, &tm);
+      cached_localtime(&now_tv_sec, &tm);
       if (!scan_iso_timestamp((const gchar **) &src, &left, &tm))
         {
           goto error;
@@ -315,8 +316,8 @@ log_msg_parse_date(LogMessage *self, const guchar **data, gint *length, guint pa
         {
           /* PIX timestamp, expected format: MMM DD YYYY HH:MM:SS: */
           /* ASA timestamp, expected format: MMM DD YYYY HH:MM:SS */
-
-          cached_localtime(&now.tv_sec, &tm);
+          time_t now_tv_sec = (time_t)now.tv_sec;
+          cached_localtime(&now_tv_sec, &tm);
           if (!scan_pix_timestamp((const gchar **) &src, &left, &tm))
             goto error;
 
@@ -338,7 +339,8 @@ log_msg_parse_date(LogMessage *self, const guchar **data, gint *length, guint pa
         {
           /* LinkSys timestamp, expected format: MMM DD HH:MM:SS YYYY */
 
-          cached_localtime(&now.tv_sec, &tm);
+          time_t now_tv_sec = (time_t)now.tv_sec;
+          cached_localtime(&now_tv_sec, &tm);
           if (!scan_linksys_timestamp((const gchar **) &src, &left, &tm))
             goto error;
           tm.tm_isdst = -1;
@@ -354,8 +356,8 @@ log_msg_parse_date(LogMessage *self, const guchar **data, gint *length, guint pa
           /* RFC 3164 timestamp, expected format: MMM DD HH:MM:SS ... */
           struct tm nowtm;
           glong usec = 0;
-
-          cached_localtime(&now.tv_sec, &nowtm);
+          time_t now_tv_sec = (time_t)now.tv_sec;
+          cached_localtime(&now_tv_sec, &nowtm);
           tm = nowtm;
           if (!scan_bsd_timestamp((const gchar **) &src, &left, &tm))
             goto error;
@@ -661,7 +663,6 @@ log_msg_parse_sd(LogMessage *self, const guchar **data, gint *length, const MsgF
   gchar sd_param_value[options->sdata_param_value_max + 1];
   gsize sd_param_value_len;
   gchar sd_value_name[66];
-  NVHandle handle;
 
   guint open_sd = 0;
   gint left = *length, pos;
@@ -714,10 +715,7 @@ log_msg_parse_sd(LogMessage *self, const guchar **data, gint *length, const MsgF
           strncpy(sd_value_name + logmsg_sd_prefix_len, sd_id_name, sizeof(sd_value_name) - logmsg_sd_prefix_len);
           if (*src == ']')
             {
-              /* Standalone sdata */
-              handle = log_msg_get_value_handle(sd_value_name);
-
-              log_msg_set_value(self, handle, "", 0);
+              log_msg_set_value_by_name(self, sd_value_name, "", 0);
             }
           else
             {
@@ -811,9 +809,7 @@ log_msg_parse_sd(LogMessage *self, const guchar **data, gint *length, const MsgF
                   goto error;
                 }
 
-              handle = log_msg_get_value_handle(sd_value_name);
-
-              log_msg_set_value(self, handle, sd_param_value, sd_param_value_len);
+              log_msg_set_value_by_name(self, sd_value_name, sd_param_value, sd_param_value_len);
             }
 
           if (left && *src == ']')
@@ -879,7 +875,7 @@ log_msg_parse_legacy(const MsgFormatOptions *parse_options,
   log_msg_parse_seq(self, &src, &left);
   log_msg_parse_skip_chars(self, &src, &left, " ", -1);
   cached_g_current_time(&now);
-  if (log_msg_parse_date(self, &src, &left, parse_options->flags & ~LP_SYSLOG_PROTOCOL, time_zone_info_get_offset(parse_options->recv_time_zone_info, now.tv_sec)))
+  if (log_msg_parse_date(self, &src, &left, parse_options->flags & ~LP_SYSLOG_PROTOCOL, time_zone_info_get_offset(parse_options->recv_time_zone_info, (time_t)now.tv_sec)))
     {
       /* Expected format: hostname program[pid]: */
       /* Possibly: Message forwarded from hostname: ... */
@@ -1091,28 +1087,7 @@ syslog_format_handler(const MsgFormatOptions *parse_options,
 
   if (G_UNLIKELY(!success))
     {
-      gchar buf[2048];
-
-      self->timestamps[LM_TS_STAMP] = self->timestamps[LM_TS_RECVD];
-      if ((self->flags & LF_STATE_OWN_PAYLOAD) && self->payload)
-        nv_table_unref(self->payload);
-      self->flags |= LF_STATE_OWN_PAYLOAD;
-      self->payload = nv_table_new(LM_V_MAX, 16, MAX(length * 2, 256));
-      log_msg_set_value(self, LM_V_HOST, "", 0);
-
-      g_snprintf(buf, sizeof(buf), "Error processing log message: %.*s", (gint) length, data);
-      log_msg_set_value(self, LM_V_MESSAGE, buf, -1);
-      log_msg_set_value(self, LM_V_PROGRAM, "syslog-ng", 9);
-      g_snprintf(buf, sizeof(buf), "%d", (int) getpid());
-      log_msg_set_value(self, LM_V_PID, buf, -1);
-
-      if (self->sdata)
-        {
-          g_free(self->sdata);
-          self->alloc_sdata = self->num_sdata = 0;
-          self->sdata = NULL;
-        }
-      self->pri = LOG_SYSLOG | LOG_ERR;
+      msg_format_inject_parse_error(self, data, length);
       return;
     }
 
