@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012 BalaBit IT Ltd, Budapest, Hungary
+ * Copyright (c) 2012-2013 BalaBit IT Ltd, Budapest, Hungary
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -21,7 +21,8 @@
  */
 
 #include "plugin.h"
-#include "templates.h"
+#include "plugin-types.h"
+#include "template/templates.h"
 #include "messages.h"
 #include "cfg.h"
 #include "tls-support.h"
@@ -43,7 +44,7 @@ TLS_BLOCK_END;
 
 #define local_state __tls_deref(geoip_state)
 
-static void
+static inline void
 tf_geoip_init(void)
 {
   if (!local_state)
@@ -58,6 +59,8 @@ tf_geoip(LogMessage *msg, gint argc, GString *argv[], GString *result)
 {
   const char *country;
 
+  tf_geoip_init();
+
   if (argc != 1)
     {
       msg_debug("tfgeoip takes only one argument",
@@ -82,7 +85,6 @@ static Plugin tfgeoip_plugins[] =
 gboolean
 tfgeoip_module_init(GlobalConfig *cfg, CfgArgs *args)
 {
-  tf_geoip_init();
   plugin_register(cfg, tfgeoip_plugins, G_N_ELEMENTS(tfgeoip_plugins));
   return TRUE;
 }
